# -*- coding: utf-8 -*-

from ..qcloudsdkcore.request import Request

class DeleteKeyPairRequest(Request):

    def __init__(self):
        super(DeleteKeyPairRequest, self).__init__(
            'cvm', 'qcloudcliV1', 'DeleteKeyPair', 'cvm.api.qcloud.com')

    def get_keyIds(self):
        return self.get_params().get('keyIds')

    def set_keyIds(self, keyIds):
        self.add_param('keyIds', keyIds)



# -*- coding: utf-8 -*-

from ..qcloudsdkcore.request import Request

class RunInstancesRequest(Request):

    def __init__(self):
        super(RunInstancesRequest, self).__init__(
            'cvm', 'qcloudcliV1', 'RunInstances', 'cvm.api.qcloud.com')

    def get_bandwidth(self):
        return self.get_params().get('bandwidth')

    def set_bandwidth(self, bandwidth):
        self.add_param('bandwidth', bandwidth)

    def get_bandwidthType(self):
        return self.get_params().get('bandwidthType')

    def set_bandwidthType(self, bandwidthType):
        self.add_param('bandwidthType', bandwidthType)

    def get_clientToken(self):
        return self.get_params().get('clientToken')

    def set_clientToken(self, clientToken):
        self.add_param('clientToken', clientToken)

    def get_cpu(self):
        return self.get_params().get('cpu')

    def set_cpu(self, cpu):
        self.add_param('cpu', cpu)

    def get_goodsNum(self):
        return self.get_params().get('goodsNum')

    def set_goodsNum(self, goodsNum):
        self.add_param('goodsNum', goodsNum)

    def get_hostName(self):
        return self.get_params().get('hostName')

    def set_hostName(self, hostName):
        self.add_param('hostName', hostName)

    def get_imageId(self):
        return self.get_params().get('imageId')

    def set_imageId(self, imageId):
        self.add_param('imageId', imageId)

    def get_imageType(self):
        return self.get_params().get('imageType')

    def set_imageType(self, imageType):
        self.add_param('imageType', imageType)

    def get_instanceName(self):
        return self.get_params().get('instanceName')

    def set_instanceName(self, instanceName):
        self.add_param('instanceName', instanceName)

    def get_instanceType(self):
        return self.get_params().get('instanceType')

    def set_instanceType(self, instanceType):
        self.add_param('instanceType', instanceType)

    def get_isVpcGateway(self):
        return self.get_params().get('isVpcGateway')

    def set_isVpcGateway(self, isVpcGateway):
        self.add_param('isVpcGateway', isVpcGateway)

    def get_keyId(self):
        return self.get_params().get('keyId')

    def set_keyId(self, keyId):
        self.add_param('keyId', keyId)

    def get_mem(self):
        return self.get_params().get('mem')

    def set_mem(self, mem):
        self.add_param('mem', mem)

    def get_needMonitorAgent(self):
        return self.get_params().get('needMonitorAgent')

    def set_needMonitorAgent(self, needMonitorAgent):
        self.add_param('needMonitorAgent', needMonitorAgent)

    def get_needSecurityAgent(self):
        return self.get_params().get('needSecurityAgent')

    def set_needSecurityAgent(self, needSecurityAgent):
        self.add_param('needSecurityAgent', needSecurityAgent)

    def get_password(self):
        return self.get_params().get('password')

    def set_password(self, password):
        self.add_param('password', password)

    def get_period(self):
        return self.get_params().get('period')

    def set_period(self, period):
        self.add_param('period', period)

    def get_privateIpAddresses(self):
        return self.get_params().get('privateIpAddresses')

    def set_privateIpAddresses(self, privateIpAddresses):
        self.add_param('privateIpAddresses', privateIpAddresses)

    def get_projectId(self):
        return self.get_params().get('projectId')

    def set_projectId(self, projectId):
        self.add_param('projectId', projectId)

    def get_purchaseSource(self):
        return self.get_params().get('purchaseSource')

    def set_purchaseSource(self, purchaseSource):
        self.add_param('purchaseSource', purchaseSource)

    def get_rootSize(self):
        return self.get_params().get('rootSize')

    def set_rootSize(self, rootSize):
        self.add_param('rootSize', rootSize)

    def get_storageSize(self):
        return self.get_params().get('storageSize')

    def set_storageSize(self, storageSize):
        self.add_param('storageSize', storageSize)

    def get_storageType(self):
        return self.get_params().get('storageType')

    def set_storageType(self, storageType):
        self.add_param('storageType', storageType)

    def get_subnetId(self):
        return self.get_params().get('subnetId')

    def set_subnetId(self, subnetId):
        self.add_param('subnetId', subnetId)

    def get_unlimitZone(self):
        return self.get_params().get('unlimitZone')

    def set_unlimitZone(self, unlimitZone):
        self.add_param('unlimitZone', unlimitZone)

    def get_vpcId(self):
        return self.get_params().get('vpcId')

    def set_vpcId(self, vpcId):
        self.add_param('vpcId', vpcId)

    def get_wanIp(self):
        return self.get_params().get('wanIp')

    def set_wanIp(self, wanIp):
        self.add_param('wanIp', wanIp)

    def get_zoneId(self):
        return self.get_params().get('zoneId')

    def set_zoneId(self, zoneId):
        self.add_param('zoneId', zoneId)



# -*- coding: utf-8 -*-

from ..qcloudsdkcore.request import Request

class InquiryPriceResetInstanceRequest(Request):

    def __init__(self):
        super(InquiryPriceResetInstanceRequest, self).__init__(
            'cvm', 'qcloudcliV1', 'InquiryPriceResetInstance', 'cvm.api.qcloud.com')

    def get_DryRun(self):
        return self.get_params().get('DryRun')

    def set_DryRun(self, DryRun):
        self.add_param('DryRun', DryRun)

    def get_EnhancedService(self):
        return self.get_params().get('EnhancedService')

    def set_EnhancedService(self, EnhancedService):
        self.add_param('EnhancedService', EnhancedService)

    def get_ImageId(self):
        return self.get_params().get('ImageId')

    def set_ImageId(self, ImageId):
        self.add_param('ImageId', ImageId)

    def get_InstanceId(self):
        return self.get_params().get('InstanceId')

    def set_InstanceId(self, InstanceId):
        self.add_param('InstanceId', InstanceId)

    def get_LoginSettings(self):
        return self.get_params().get('LoginSettings')

    def set_LoginSettings(self, LoginSettings):
        self.add_param('LoginSettings', LoginSettings)

    def get_SystemDisk(self):
        return self.get_params().get('SystemDisk')

    def set_SystemDisk(self, SystemDisk):
        self.add_param('SystemDisk', SystemDisk)



# -*- coding: utf-8 -*-

from ..qcloudsdkcore.request import Request

class CreateBmLoadBalancerRequest(Request):

    def __init__(self):
        super(CreateBmLoadBalancerRequest, self).__init__(
            'bmlb', 'qcloudcliV1', 'CreateBmLoadBalancer', 'bmlb.api.qcloud.com')

    def get_exclusive(self):
        return self.get_params().get('exclusive')

    def set_exclusive(self, exclusive):
        self.add_param('exclusive', exclusive)

    def get_goodsNum(self):
        return self.get_params().get('goodsNum')

    def set_goodsNum(self, goodsNum):
        self.add_param('goodsNum', goodsNum)

    def get_loadBalancerType(self):
        return self.get_params().get('loadBalancerType')

    def set_loadBalancerType(self, loadBalancerType):
        self.add_param('loadBalancerType', loadBalancerType)

    def get_payMode(self):
        return self.get_params().get('payMode')

    def set_payMode(self, payMode):
        self.add_param('payMode', payMode)

    def get_projectId(self):
        return self.get_params().get('projectId')

    def set_projectId(self, projectId):
        self.add_param('projectId', projectId)

    def get_specifiedVips(self):
        return self.get_params().get('specifiedVips')

    def set_specifiedVips(self, specifiedVips):
        self.add_param('specifiedVips', specifiedVips)

    def get_tgwSetType(self):
        return self.get_params().get('tgwSetType')

    def set_tgwSetType(self, tgwSetType):
        self.add_param('tgwSetType', tgwSetType)

    def get_unSubnetId(self):
        return self.get_params().get('unSubnetId')

    def set_unSubnetId(self, unSubnetId):
        self.add_param('unSubnetId', unSubnetId)

    def get_unVpcId(self):
        return self.get_params().get('unVpcId')

    def set_unVpcId(self, unVpcId):
        self.add_param('unVpcId', unVpcId)

    def get_vipIspId(self):
        return self.get_params().get('vipIspId')

    def set_vipIspId(self, vipIspId):
        self.add_param('vipIspId', vipIspId)



# -*- coding: utf-8 -*-

from ..qcloudsdkcore.request import Request

class DeleteKeyPairsRequest(Request):

    def __init__(self):
        super(DeleteKeyPairsRequest, self).__init__(
            'cvm', 'qcloudcliV1', 'DeleteKeyPairs', 'cvm.api.qcloud.com')

    def get_KeyIds(self):
        return self.get_params().get('KeyIds')

    def set_KeyIds(self, KeyIds):
        self.add_param('KeyIds', KeyIds)



# -*- coding: utf-8 -*-

from ..qcloudsdkcore.request import Request

class ImportKeyPairRequest(Request):

    def __init__(self):
        super(ImportKeyPairRequest, self).__init__(
            'cvm', 'qcloudcliV1', 'ImportKeyPair', 'cvm.api.qcloud.com')

    def get_keyName(self):
        return self.get_params().get('keyName')

    def set_keyName(self, keyName):
        self.add_param('keyName', keyName)

    def get_pubKey(self):
        return self.get_params().get('pubKey')

    def set_pubKey(self, pubKey):
        self.add_param('pubKey', pubKey)



# -*- coding: utf-8 -*-

from ..qcloudsdkcore.request import Request

class RestartInstancesRequest(Request):

    def __init__(self):
        super(RestartInstancesRequest, self).__init__(
            'cvm', 'qcloudcliV1', 'RestartInstances', 'cvm.api.qcloud.com')

    def get_forceStop(self):
        return self.get_params().get('forceStop')

    def set_forceStop(self, forceStop):
        self.add_param('forceStop', forceStop)

    def get_instanceIds(self):
        return self.get_params().get('instanceIds')

    def set_instanceIds(self, instanceIds):
        self.add_param('instanceIds', instanceIds)



#!/usr/bin/env python
# coding: utf-8

from mako.lookup import TemplateLookup

from sa_tools_core.consts import PROJECT_ROOT
from sa_tools_core.utils import to_str


TEMPLATES_DIR = PROJECT_ROOT + '/templates'
MAGIC_EMPTY_LINE_MARK = '|'
lookup = TemplateLookup(directories=[TEMPLATES_DIR],
                        input_encoding='utf8',
                        output_encoding='utf8',
                        encoding_errors='ignore')


def render(template_name, strip_empty_lines=True, **data):
    out = lookup.get_template(template_name).render(**data)
    # When using Python 3, the render() method will return a bytes object, if output_encoding is set. Otherwise it returns
    # a string.
    out = to_str(out)

    if strip_empty_lines:
        return '\n'.join([('' if i.strip() == MAGIC_EMPTY_LINE_MARK else i)
                         for i in out.splitlines() if i.strip()]) + '\n'
    else:
        return out


def render_notification(**kw):
    title = render('icinga/notification.mako',
                   fragment_type='title',
                   **kw)
    content = render('icinga/notification.mako',
                     fragment_type='content',
                     **kw)
    return title, content



# -*- coding: utf-8 -*-

from ..qcloudsdkcore.request import Request

class DescribeBmVpcExRequest(Request):

    def __init__(self):
        super(DescribeBmVpcExRequest, self).__init__(
            'bmvpc', 'qcloudcliV1', 'DescribeBmVpcEx', 'bmvpc.api.qcloud.com')

    def get_limit(self):
        return self.get_params().get('limit')

    def set_limit(self, limit):
        self.add_param('limit', limit)

    def get_offset(self):
        return self.get_params().get('offset')

    def set_offset(self, offset):
        self.add_param('offset', offset)

    def get_orderDirection(self):
        return self.get_params().get('orderDirection')

    def set_orderDirection(self, orderDirection):
        self.add_param('orderDirection', orderDirection)

    def get_orderField(self):
        return self.get_params().get('orderField')

    def set_orderField(self, orderField):
        self.add_param('orderField', orderField)

    def get_unVpcId(self):
        return self.get_params().get('unVpcId')

    def set_unVpcId(self, unVpcId):
        self.add_param('unVpcId', unVpcId)

    def get_vpcId(self):
        return self.get_params().get('vpcId')

    def set_vpcId(self, vpcId):
        self.add_param('vpcId', vpcId)

    def get_vpcName(self):
        return self.get_params().get('vpcName')

    def set_vpcName(self, vpcName):
        self.add_param('vpcName', vpcName)

    def get_vpcStatus(self):
        return self.get_params().get('vpcStatus')

    def set_vpcStatus(self, vpcStatus):
        self.add_param('vpcStatus', vpcStatus)

    def get_zoneIds(self):
        return self.get_params().get('zoneIds')

    def set_zoneIds(self, zoneIds):
        self.add_param('zoneIds', zoneIds)



# coding: utf-8

from __future__ import print_function

import sys
import json
import logging
import argparse
import importlib
import subprocess
from functools import partial
from collections import defaultdict

from sa_tools_core.utils import get_os_username, jprint
from sa_tools_core.consts import (BS_API_REQUESTS_MODULE_PREFIX,
                                  BS_CMD_PATTERN,
                                  BS_DEFAULT_ATTRS,
                                  BS_DEFAULT_PARAMS,
                                  BS_DEFAULT_PARAMS_BM,
                                  BS_PLURAL_SUFFIX)

logger = logging.getLogger(__name__)

"""
参数输入处理规则
instanceId -> alias
subnetId -> subnetName -> cidrBlock
vpcId -> vpcName -> cidrBlock
eipId -> eipName -> eip
vpcIp == ip

复杂结构 params:
  list、dict
list params input, don't use json
  ip -> ipList, ips
  instanceId -> instanceIds
若 list 跟 dict 嵌套，将 list 抹掉: subnetSet
"""

# TODO: support Chinese help message (但 qcloudcli 跟这个文档并不完全对应)
# https://cloud.tencent.com/document/api/386/6632

action_mapping = defaultdict(dict)

# TODO: add PARAM_MAPPING for cvm
PARAM_MAPPING = {
    'instanceId': (['alias'], ['get_data_for_reduce', 'bm', 'list'],
                   lambda x: [i['instanceId'] for i in x[0].get('deviceList', []) if i['alias'] == x[1]['alias']][0],
                   ),
    'subnetId': (['subnetName'], ['get_data_for_reduce', 'bmvpc', 'subnet'],
                 lambda x: [i['subnetId'] for i in x[0] if i['subnetName'] == x[1]['subnetName']][0],
                 ),
    'unSubnetId': (['subnetName'], ['get_data_for_reduce', 'bmvpc', 'subnet'],
                   lambda x: [i['unSubnetId'] for i in x[0] if i['subnetName'] == x[1]['subnetName']][0],
                   ),
    'vpcId': (['vpcName'], ['get_data_for_reduce', 'bmvpc', 'list'],
              lambda x: [i['vpcId'] for i in x[0] if i['vpcName'] == x[1]['vpcName']][0],
              ),
    # NOTE: eips -> eipIds; --eip is not valid, will get all eips, but result will be fine if there're not many eips.
    #   we hack this in get_data_for_reduce.
    'eipId': (['eip'], ['get_data_for_reduce', 'bmeip', 'list'],
              lambda x: [i['eipId'] for i in x[0].get('eipSet', []) if i['eip'] == x[1]['eip']][0],
              ),

    'ipList': (['ip'], ['list2json']),
    'ips': (['ip'], ['list2json']),
    'instanceIds': (['instanceId'], ['list2json']),
    'eipIds': (['eipId'], ['list2json']),
    'eips': (['eip'], ['list2json']),
    'aliases': (['alias'], ['list2json']),

    'subnetSet': (['subnetName', 'cidrBlock'], ['list2json']),
}


def translate_param(args, param_key, new_params=None):
    logger.debug('translate_param(args=%s, param_key=%s, new_params=%s)', args, param_key, new_params)
    param_translate_rule = PARAM_MAPPING[param_key]

    # e.g. instanceIds -> instanceId -> alias
    if len(param_translate_rule[0]) == 1:
        intermediate_param_key = param_translate_rule[0][0]
        if intermediate_param_key in PARAM_MAPPING and getattr(args, intermediate_param_key) is None:
            _param_key = PARAM_MAPPING[intermediate_param_key][0][0]
            if isinstance(getattr(args, _param_key), list):
                rets = [translate_param(args, intermediate_param_key, {_param_key: i})
                        for i in getattr(args, _param_key)]
                new_params = {intermediate_param_key: rets}
                if all((v is None or isinstance(v, list) and all(i is None for i in v)) for v in new_params.values()):
                    new_params = None

    if new_params is None:
        new_params = {k: getattr(args, k) for k in param_translate_rule[0]}
    if all(v is None for v in new_params.values()):
        return

    def _func(r, f):
        if not isinstance(f, list):
            f = [f]
        _f = f[0]
        if not callable(_f):
            _f = globals()[_f]
        return partial(_f, *f[1:])(r)

    try:
        ret = reduce(_func, param_translate_rule[1:], new_params)
    except IndexError:
        return
    return ret


def list2json(params):
    logger.debug('list2json(params=%s)', params)
    if len(params) == 1:
        obj = params.values()[0]
    else:
        n = len(params.values()[0])
        assert all(len(v) == n for v in params.values())
        obj = [{k: v[i] for k, v in params.items() if v[i] is not None} for i in range(n)]
    if not obj:
        return
    return json.dumps(obj).replace('"', '\\"')


def action_simplify(action, mod_suffix=''):
    mod_suffix = mod_suffix.lstrip('bm')
    if mod_suffix:
        mod_suffix = mod_suffix[0].upper() + mod_suffix[1:]
    action = (action.replace('Get', '')
                    .replace('Describe', '')
                    .replace('Description', '')
                    .replace('UnBind', 'Unbind')
                    .replace('LoadBalancer', 'Lb')
                    .replace('Ex', '')
                    .replace(mod_suffix, '')
                    .replace('Bm', '')
                    .replace('Query', ''))
    action = ''.join(['_%s' % c.lower() if c.isupper() else c for c in action])[1:]
    return action


def populate_subparser(parser, mod_suffix=''):
    mod_suffix = mod_suffix.lower()
    parser.set_defaults(mod_suffix=mod_suffix)

    module_path = BS_API_REQUESTS_MODULE_PREFIX + mod_suffix
    mod = importlib.import_module(module_path)
    mod_suffix = mod_suffix if mod_suffix.lstrip('bm') else 'device'
    mapping = action_mapping[mod_suffix]
    for action in dir(mod):
        if action.endswith('Request'):
            orig_action = action[:-len('Request')]
            simple_action = action_simplify(orig_action, mod_suffix=mod_suffix)
            mapping[simple_action or 'list'] = orig_action

    subparsers = parser.add_subparsers(help='Sub commands')
    for simple_action, orig_action in mapping.items():
        action_parser = subparsers.add_parser(simple_action, help='')
        action_parser.add_argument('-r', '--raw', action='store_true', help='raw output.')
        action_parser.add_argument('-j', '--json', action='store_true', help='json format output.')
        action_parser.add_argument('-a', '--attrs', nargs='*', default=list(BS_DEFAULT_ATTRS),
                                   help='the attrs that should be output. (default: %(default)s)')
        action_parser.add_argument('-e', '--extra-attrs', nargs='*',
                                   help='the extra-attrs that should be output.')
        action_parser.add_argument('-s', '--sep', default=', ', help='separator, (default: %(default)s)')

        action_parser.set_defaults(orig_action=orig_action)
        action_mod = getattr(mod, orig_action + 'Request')
        action_cls = getattr(action_mod, orig_action + 'Request')
        param_keys = {param[len('set_'):] for param in action_cls.__dict__ if param.startswith('set_')}
        action_parser.set_defaults(param_keys=param_keys)
        translate_param_keys = set()
        translate_added_param_keys = set()
        extra_new_param_keys = defaultdict(list)
        logger.debug('mod: %s, action: %s, param_keys: %s', mod_suffix, simple_action, param_keys)
        for param_key in param_keys:
            # original params
            kw = {}
            if mod_suffix.startswith('bm') and param_key in BS_DEFAULT_PARAMS_BM:
                kw['default'] = BS_DEFAULT_PARAMS_BM[param_key]
                kw['help'] = '(default: %(default)s)'
            elif param_key in BS_DEFAULT_PARAMS:
                kw['default'] = BS_DEFAULT_PARAMS[param_key]
                kw['help'] = '(default: %(default)s)'
            action_parser.add_argument('--%s' % param_key, **kw)

            # translated params
            if param_key in PARAM_MAPPING:
                new_param_keys = PARAM_MAPPING[param_key][0]

                if len(new_param_keys) == 1 and new_param_keys[0] in PARAM_MAPPING:
                    extra_new_param_keys[param_key] += PARAM_MAPPING[new_param_keys[0]][0]

                if all(new_param_key not in param_keys for new_param_key in new_param_keys):
                    translate_param_keys.add(param_key)
                    kw = {}
                    kw['help'] = 'translated param -> %s' % param_key
                    if any(param_key.endswith(suffix) for suffix in BS_PLURAL_SUFFIX):
                        kw['nargs'] = '*'
                    for new_param_key in new_param_keys:
                        if new_param_key in translate_added_param_keys:
                            continue
                        try:
                            action_parser.add_argument('--%s' % new_param_key, **kw)
                            logger.debug('add new_param_key: %s', new_param_key)
                            translate_added_param_keys.add(new_param_key)
                        except Exception:
                            logger.error('param_keys: %s, translate_added_param_keys: %s, new_param_key: %s',
                                         param_keys, translate_added_param_keys, new_param_key)
                            raise
                else:
                    logger.debug('some new_param_keys(%s) already be added in param_keys(%s).',
                                 new_param_keys, param_keys)

        # extra translated params
        for param_key, extra_new_param_keys in extra_new_param_keys.items():
            kw = {}
            kw['help'] = 'translated param -> %s' % param_key
            if any(param_key.endswith(suffix) for suffix in BS_PLURAL_SUFFIX):
                kw['nargs'] = '*'
            for extra_new_param_key in extra_new_param_keys:
                if extra_new_param_key not in (param_keys | translate_added_param_keys):
                    try:
                        action_parser.add_argument('--%s' % extra_new_param_key, **kw)
                        logger.debug('add extra_new_param_key: %s', extra_new_param_key)
                    except Exception:
                        logger.error('param_keys: %s, translate_added_param_keys: %s, extra_new_param_key: %s',
                                     param_keys, translate_added_param_keys, extra_new_param_key)
                        raise

        action_parser.set_defaults(translate_param_keys=translate_param_keys)


def _str(i):
    s = None
    try:
        s = str(i)
    except UnicodeEncodeError:
        s = str(i.encode('utf-8'))
    return s


def output_simplify(args, output):
    try:
        ret = json.loads(output)
    except ValueError:
        logger.error('output: %s', output)
        raise
    code = ret.get('code')
    if code != 0:
        logger.warn('response code non-zero')
        return False, ret

    data = ret.get('data')
    # FIXME: some api output no "data" dict key
    if not data:
        _ret = {k: v for k, v in ret.items() if k.endswith('Set')}
        if _ret:
            data = _ret
    attrs = set(args.attrs + (args.extra_attrs or []))

    def _simplify(data):
        if isinstance(data, list):
            data = [_simplify(i) for i in data]
            if not args.json:
                data = '\n'.join(sorted(_str(i) for i in data))
        elif isinstance(data, dict):
            is_leaf = False
            # NOTE: rough leaf determination
            if not any(isinstance(v, (dict,)) for k, v in data.items()):
                is_leaf = True
            if any(isinstance(i, (dict,)) for k, v in data.items() if isinstance(v, list) for i in v):
                is_leaf = False
            data = {k: _simplify(v) for k, v in data.items() if not is_leaf or k in attrs} or data
            if not args.json:
                if is_leaf:
                    data = args.sep.join([_str(v) for k, v in data.items()])
                else:
                    data = '\n'.join([('%s' if idx % 2 else '>> %s') % _str(i) for p in data.items() for idx, i in enumerate(p)])
        return data

    return True, _simplify(data)


# TODO: support direct API call
def _execute(mod_suffix, action, params):
    logger.debug('call _execute(%s, %s, %s)', mod_suffix, action, params)

    mapping = action_mapping[mod_suffix if mod_suffix.lstrip('bm') else 'device']
    if action in mapping:
        action = mapping[action]

    if isinstance(params, dict):
        params = params.items()
    params = ' '.join(['--"%s" "%s"' % (k, v) for k, v in params if v is not None])
    cmd = BS_CMD_PATTERN.format(module=mod_suffix,
                                action=action,
                                params=params)
    logger.info('CMD: %s', cmd)
    p = subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    stdout, stderr = p.communicate()
    if p.returncode != 0 or stderr:
        logger.error('CMD ERROR: %s', stderr)
    return stdout


def get_data_for_reduce(mod_suffix, action, params):
    # HACK: eip list need eips, but we got eip, so we translate it here
    if (mod_suffix, action) == ('bmeip', 'list'):
        if PARAM_MAPPING['eips'][0][0] in params.keys():
            params.update({'eips': list2json(
                {'eips': [params['eip']]}
                )})

    output = _execute(mod_suffix, action, params)
    data = json.loads(output).get('data', {})
    logger.debug('get_data_for_reduce: data: %s, params: %s', data, params)
    return data, params


def execute(args):
    params = [(k, getattr(args, k)) for k in args.param_keys if getattr(args, k) is not None]
    for k in args.param_keys:
        if k in PARAM_MAPPING and k in args.translate_param_keys and getattr(args, k) is None:
            logger.debug('%s should be translated to %s', k, PARAM_MAPPING[k][0])
            translated_param = translate_param(args, k)
            if translated_param is not None:
                params.append((k, translated_param))
                logger.debug('translated success: %s -> %s=%s', PARAM_MAPPING[k][0], k, translated_param)

    output = _execute(args.mod_suffix, action=args.orig_action, params=params)
    if args.raw:
        return True, output
    return output_simplify(args, output)


def device(args):
    """ bm device """
    ret, output = execute(args)
    if output:
        if args.json or ret is False:
            jprint(output)
        else:
            print(output)


cvm = bmeip = bmlb = bmvpc = device


def main():
    """
    e.g.

    API DOC: https://cloud.tencent.com/document/api/386/6632

    $ sa-bs device list -j
    $ sa-bs device list -a alias
    $ sa-bs device list --alias host
    $ sa-bs vpc list -e createTime vpcId
    $ sa-bs vpc subnet
    $ sa-bs vpc subnet_ips --vpcId 1001 --subnetId 6555 -j
    $ sa-bs vpc subnet_ips --subnetName SA
    $ sa-bs vpc subnet_by_cpm_id --alias host22
    $ sa-bs eip list -a eip
    $ sa-bs lb list
    $ sa-bs -vvvv eip list --eipIds '[\\"eip-xxxxxxxx\\"]' -r
    $ sa-bs eip list --eip 1.1.1.1
    $ sa-bs vpc register_batch_ip --subnetName SA --ip 10.0.0.1
    $ sa-bs eip apply
    $ sa-bs eip bind_vpc_ip --eip 1.1.1.1 --vpcIp 10.0.0.1
    $ sa-bs vpc create_interface --alias host11 host22 --subnetName DBA-dummy
    $ sa-bs device reload_os --passwd XXXXXX --subnetName OfflineComputation --alias host88
    $ sa-bs device modify_alias --alias host33 --instanceId cpm-xxxxxxxx
    $ sa-bs -vvvv vpc create_subnet --subnetName Isolation-dummy --cidrBlock 10.0.1.0/24 --vlanId 2222

    ## 机型组合
    $ sa-bs device list -e deviceClassCode
    $ sa-bs device os --deviceClassCode Y0-BS09v2 -a osNameDisplay osTypeId
    $ sa-bs device class_partition --cpuId 4 --diskNum1 2 --diskNum2 12 --diskTypeId1 1 --diskTypeId2 6 --haveRaidCard 0 --mem 64 --deviceClassCode "Y0-BS09v2"
    $ sa-bs device class_partition --cpuId 4 --diskNum1 2 --diskNum2 12 --diskTypeId1 1 --diskTypeId2 6 --haveRaidCard 0 --mem 64
    $ sa-bs device elastic_price --cpuId 4 --diskNum1 2 --diskNum2 12 --diskTypeId1 1 --diskTypeId2 6 --haveRaidCard 0 --mem 64
    $ sa-bs device inventory --cpuId 4 --diskNum1 2 --diskNum2 12 --diskTypeId1 1 --diskTypeId2 6 --haveRaidCard 0 --mem 64 --deviceClassCode "Y0-BS09v2" --subnetName OfflineComputation
    $ sa-bs device hardware_info --alias host11
    $ sa-bs device hardware_specification

    ### 购买机器 ( see https://cloud.tencent.com/document/api/386/6638 )
    $ sa-bs device buy --goodsNum 2 --timeSpan 1 --timeUnit m --alias new_host \
        --subnetName SA --ip 10.0.0.2 10.0.0.3 \
        --cpuId 4 --diskNum1 2 --diskNum2 12 --diskTypeId1 1 --diskTypeId2 6 --haveRaidCard 0 --mem 64 \
        --raidId 25 \
        --deviceClassCode "Y0-BS09v2" --needSecurityAgent 0 --needMonitorAgent 0 --autoRenewFlag 1
    $ sa-bs device deploy_process --instanceId cpm-xxxxxxxx
    $ sa-bs device deploy_process --alias host11
    $ sa-bs device operation_log --alias host22

    ## CVM

    $ sa-bs cvm instances

    """  # NOQA
    parser = argparse.ArgumentParser(epilog=main.__doc__,
                                     formatter_class=argparse.RawDescriptionHelpFormatter)
    parser.add_argument('-u', '--user', help='LDAP username, use your OS user name by default.')
    parser.add_argument('-v', '--verbose', action='count', default=0, help='verbose.')

    # compatible with py2 & 3
    subparsers = parser.add_subparsers(help='Sub commands', dest='subparser')
    subparsers.required = True

    # level = logging.DEBUG
    # logging.basicConfig(level=level,
    #                     format='%(asctime)s %(name)s %(levelname)s %(message)s')

    device_parser = subparsers.add_parser('device', help='')
    populate_subparser(device_parser, mod_suffix='bm')
    device_parser.set_defaults(func=device)

    eip_parser = subparsers.add_parser('eip', help='')
    populate_subparser(eip_parser, mod_suffix='bmEip')
    eip_parser.set_defaults(func=bmeip)

    lb_parser = subparsers.add_parser('lb', help='')
    populate_subparser(lb_parser, mod_suffix='bmLb')
    lb_parser.set_defaults(func=bmlb)

    vpc_parser = subparsers.add_parser('vpc', help='')
    populate_subparser(vpc_parser, mod_suffix='bmVpc')
    vpc_parser.set_defaults(func=bmvpc)

    cvm_parser = subparsers.add_parser('cvm', help='')
    populate_subparser(cvm_parser, mod_suffix='Cvm')
    cvm_parser.set_defaults(func=cvm)

    args = parser.parse_args()
    args.user = args.user or get_os_username()

    level = logging.WARNING - args.verbose * 10
    logging.basicConfig(level=level,
                        format='%(asctime)s %(name)s %(levelname)s %(message)s')

    sys.exit(args.func(args))



# coding: utf-8

import re
import sys
import logging
import argparse

from sa_tools_core.libs.permission import require_user
from sa_tools_core.libs.notify.email import send_mail
from sa_tools_core.libs.notify.wechat import send_message as send_wechat
from sa_tools_core.libs.notify.pushbullet import send_message as send_pushbullet
from sa_tools_core.libs.notify.pushover import send_message as send_pushover
from sa_tools_core.libs.notify.telegram import send_message as send_telegram
from sa_tools_core.libs.notify.sms import send_sms
from sa_tools_core.utils import get_os_username
from sa_tools_core.consts import SYSADMIN_EMAIL

logger = logging.getLogger(__name__)
logging.getLogger("requests").setLevel(logging.WARN)

NOTIFY_TYPES = ('wechat', 'wework', 'email', 'sms', 'pushbullet', 'pushover', 'telegram')
DEFAULT_TITLE = 'Sent from sa-notify'


class Notifier(object):
    def __init__(self, content=None, title=DEFAULT_TITLE, from_addr=SYSADMIN_EMAIL):
        self.content = content
        self.title = title
        self.from_addr = from_addr

    def __getattr__(self, attr):
        if attr in NOTIFY_TYPES:
            f = getattr(self, '_%s' % attr)

            def _func(*a, **kw):
                addrs = kw.get('addrs') or a[0]
                addrs = [addr for addr in addrs if addr]
                if not addrs:
                    logger.warning('notify abort, ignoring empty %s addrs', attr)
                    return
                kw['content'] = kw.get('content') or self.content
                kw['title'] = kw.get('title') or self.title
                if kw['title']:
                    kw['title'] = kw['title'].strip()
                logger.info('notify %s %s', attr, addrs)
                return f(*a, **kw)
            return _func
        raise AttributeError

    def _wechat(self, addrs, content=None, **kw):
        send_wechat('|'.join(addrs), content)

    def _wework(self, addrs, content=None, **kw):
        send_wechat('|'.join(addrs), content, qy=True)

    def _email(self, addrs, content=None, title=None, from_addr=None, **kw):
        send_mail(addrs, content, subject=title, from_addr=from_addr or self.from_addr)

    def _pushbullet(self, addrs, content=None, title=None, **kw):
        for addr in addrs:
            send_pushbullet(addr, title, content)

    def _pushover(self, addrs, content=None, **kw):
        for addr in addrs:
            send_pushover(addr, content)

    def _telegram(self, addrs, content=None, **kw):
        for addr in addrs:
            send_telegram(addr, content)

    def _sms(self, addrs, content=None, **kw):
        for addr in addrs:
            send_sms(addr, content)


@require_user
def notify(args):
    notifier = Notifier(args.content, args.subject)
    for type_ in NOTIFY_TYPES:
        values = vars(args)[type_]
        if values:
            addrs = [i for v in values for i in re.split(r'[,\s]+', v)]
            try:
                getattr(notifier, type_)(addrs, from_addr=args.from_addr)
            except Exception as e:
                logger.exception('Notifier.%s(%s) failed: %s', type_, addrs, e)


def main(args=None):
    """
    e.g.
    $ sa-notify --wechat user1 --content 'xxx'
    $ echo 'xxx' | sa-notify --wechat user1,user2 --email user1@douban.com user3@douban.com
    """
    parser = argparse.ArgumentParser(epilog=main.__doc__,
                                     formatter_class=argparse.RawDescriptionHelpFormatter)
    parser.add_argument('-u', '--user', help='LDAP username, use your OS user name by default.')
    parser.add_argument('-c', '--content', help='content.')
    parser.add_argument('-s', '--subject', help='subject. default="%(default)s"',
                        default=DEFAULT_TITLE)
    parser.add_argument('-f', '--from-addr', help='From address, currently only works for email.')
    for type_ in NOTIFY_TYPES:
        parser.add_argument('--%s' % type_, nargs='*',
                            help='your enterprise address of %s.' % type_)

    args = parser.parse_args(args)
    args.user = args.user or get_os_username()

    logging.basicConfig(level=logging.INFO,
                        format='%(asctime)s %(name)s %(levelname)s %(message)s')

    if not any(vars(args)[type_] for type_ in NOTIFY_TYPES):
        parser.error("too few arguments")
    if args.content is None:
        args.content = sys.stdin.read()
        if not args.content:
            parser.error("content empty")

    notify(args)



# -*- coding: utf-8 -*-

from ..qcloudsdkcore.request import Request

class ModifyKeyPairAttributeRequest(Request):

    def __init__(self):
        super(ModifyKeyPairAttributeRequest, self).__init__(
            'cvm', 'qcloudcliV1', 'ModifyKeyPairAttribute', 'cvm.api.qcloud.com')

    def get_Description(self):
        return self.get_params().get('Description')

    def set_Description(self, Description):
        self.add_param('Description', Description)

    def get_KeyId(self):
        return self.get_params().get('KeyId')

    def set_KeyId(self, KeyId):
        self.add_param('KeyId', KeyId)

    def get_KeyName(self):
        return self.get_params().get('KeyName')

    def set_KeyName(self, KeyName):
        self.add_param('KeyName', KeyName)



from . import (BindInstanceKeyRequest,CreateKeyPairRequest,DeleteKeyPairRequest,DescribeAvailabilityZonesRequest,DescribeInstancesRequest,DescribeKeyPairsRequest,DescribeZoneAbilityRequest,ImportKeyPairRequest,InquiryInstancePriceHourRequest,InquiryInstancePriceRequest,ModifyInstanceAttributesRequest,ModifyInstanceProjectRequest,ModifyKeyPairRequest,RenewInstanceRequest,ResetInstancePasswordRequest,ResetInstancesRequest,ResizeInstanceHourRequest,RestartInstancesRequest,ReturnInstanceRequest,RunInstancesHourRequest,RunInstancesRequest,StartInstancesRequest,StopInstancesRequest,UnBindInstanceKeyRequest,UpdateInstanceBandwidthHourRequest,UpdateInstanceBandwidthRequest,)



# -*- coding: utf-8 -*-

from ..qcloudsdkcore.request import Request

class SubnetUnBindBmNatGatewayRequest(Request):

    def __init__(self):
        super(SubnetUnBindBmNatGatewayRequest, self).__init__(
            'bmvpc', 'qcloudcliV1', 'SubnetUnBindBmNatGateway', 'bmvpc.api.qcloud.com')

    def get_natId(self):
        return self.get_params().get('natId')

    def set_natId(self, natId):
        self.add_param('natId', natId)

    def get_subnetIds(self):
        return self.get_params().get('subnetIds')

    def set_subnetIds(self, subnetIds):
        self.add_param('subnetIds', subnetIds)

    def get_unSubnetIds(self):
        return self.get_params().get('unSubnetIds')

    def set_unSubnetIds(self, unSubnetIds):
        self.add_param('unSubnetIds', unSubnetIds)

    def get_unVpcId(self):
        return self.get_params().get('unVpcId')

    def set_unVpcId(self, unVpcId):
        self.add_param('unVpcId', unVpcId)

    def get_vpcId(self):
        return self.get_params().get('vpcId')

    def set_vpcId(self, vpcId):
        self.add_param('vpcId', vpcId)



# coding: utf-8

import logging
import requests

from sa_tools_core.utils import get_config
from sa_tools_core.consts import PROXIES

logger = logging.getLogger(__name__)


def send_message(chatid, content):
    PUSH_KEY = get_config('telegram')
    PUSH_URL = 'https://api.telegram.org/bot%s/sendMessage' % PUSH_KEY

    message = {
        "chat_id": chatid,
        "text": content
    }

    req = requests.post(PUSH_URL, json=message, proxies=PROXIES)
    resp = req.json()
    if not resp['ok']:
        raise Exception('api failed, resp: %s' % resp)



# -*- coding: utf-8 -*-

from ..qcloudsdkcore.request import Request

class ModifyBmLoadBalancerAttributesRequest(Request):

    def __init__(self):
        super(ModifyBmLoadBalancerAttributesRequest, self).__init__(
            'bmlb', 'qcloudcliV1', 'ModifyBmLoadBalancerAttributes', 'bmlb.api.qcloud.com')

    def get_domainPrefix(self):
        return self.get_params().get('domainPrefix')

    def set_domainPrefix(self, domainPrefix):
        self.add_param('domainPrefix', domainPrefix)

    def get_loadBalancerId(self):
        return self.get_params().get('loadBalancerId')

    def set_loadBalancerId(self, loadBalancerId):
        self.add_param('loadBalancerId', loadBalancerId)

    def get_loadBalancerName(self):
        return self.get_params().get('loadBalancerName')

    def set_loadBalancerName(self, loadBalancerName):
        self.add_param('loadBalancerName', loadBalancerName)



# -*- coding: utf-8 -*-

from ..qcloudsdkcore.request import Request

class ModifyBmListenerRequest(Request):

    def __init__(self):
        super(ModifyBmListenerRequest, self).__init__(
            'bmlb', 'qcloudcliV1', 'ModifyBmListener', 'bmlb.api.qcloud.com')

    def get_bandwidth(self):
        return self.get_params().get('bandwidth')

    def set_bandwidth(self, bandwidth):
        self.add_param('bandwidth', bandwidth)

    def get_customHealthSwitch(self):
        return self.get_params().get('customHealthSwitch')

    def set_customHealthSwitch(self, customHealthSwitch):
        self.add_param('customHealthSwitch', customHealthSwitch)

    def get_healthNum(self):
        return self.get_params().get('healthNum')

    def set_healthNum(self, healthNum):
        self.add_param('healthNum', healthNum)

    def get_healthRequest(self):
        return self.get_params().get('healthRequest')

    def set_healthRequest(self, healthRequest):
        self.add_param('healthRequest', healthRequest)

    def get_healthResponse(self):
        return self.get_params().get('healthResponse')

    def set_healthResponse(self, healthResponse):
        self.add_param('healthResponse', healthResponse)

    def get_healthSwitch(self):
        return self.get_params().get('healthSwitch')

    def set_healthSwitch(self, healthSwitch):
        self.add_param('healthSwitch', healthSwitch)

    def get_inputType(self):
        return self.get_params().get('inputType')

    def set_inputType(self, inputType):
        self.add_param('inputType', inputType)

    def get_intervalTime(self):
        return self.get_params().get('intervalTime')

    def set_intervalTime(self, intervalTime):
        self.add_param('intervalTime', intervalTime)

    def get_lineSeparatorType(self):
        return self.get_params().get('lineSeparatorType')

    def set_lineSeparatorType(self, lineSeparatorType):
        self.add_param('lineSeparatorType', lineSeparatorType)

    def get_listenerId(self):
        return self.get_params().get('listenerId')

    def set_listenerId(self, listenerId):
        self.add_param('listenerId', listenerId)

    def get_listenerName(self):
        return self.get_params().get('listenerName')

    def set_listenerName(self, listenerName):
        self.add_param('listenerName', listenerName)

    def get_loadBalancerId(self):
        return self.get_params().get('loadBalancerId')

    def set_loadBalancerId(self, loadBalancerId):
        self.add_param('loadBalancerId', loadBalancerId)

    def get_sessionExpire(self):
        return self.get_params().get('sessionExpire')

    def set_sessionExpire(self, sessionExpire):
        self.add_param('sessionExpire', sessionExpire)

    def get_timeOut(self):
        return self.get_params().get('timeOut')

    def set_timeOut(self, timeOut):
        self.add_param('timeOut', timeOut)

    def get_toaFlag(self):
        return self.get_params().get('toaFlag')

    def set_toaFlag(self, toaFlag):
        self.add_param('toaFlag', toaFlag)

    def get_unhealthNum(self):
        return self.get_params().get('unhealthNum')

    def set_unhealthNum(self, unhealthNum):
        self.add_param('unhealthNum', unhealthNum)



# -*- coding: utf-8 -*-

from ..qcloudsdkcore.request import Request

class DescribeEipBmRequest(Request):

    def __init__(self):
        super(DescribeEipBmRequest, self).__init__(
            'bmeip', 'qcloudcliV1', 'DescribeEipBm', 'bmeip.api.qcloud.com')

    def get_aclId(self):
        return self.get_params().get('aclId')

    def set_aclId(self, aclId):
        self.add_param('aclId', aclId)

    def get_bindAcl(self):
        return self.get_params().get('bindAcl')

    def set_bindAcl(self, bindAcl):
        self.add_param('bindAcl', bindAcl)

    def get_bindTypes(self):
        return self.get_params().get('bindTypes')

    def set_bindTypes(self, bindTypes):
        self.add_param('bindTypes', bindTypes)

    def get_eipIds(self):
        return self.get_params().get('eipIds')

    def set_eipIds(self, eipIds):
        self.add_param('eipIds', eipIds)

    def get_eips(self):
        return self.get_params().get('eips')

    def set_eips(self, eips):
        self.add_param('eips', eips)

    def get_exclusiveTag(self):
        return self.get_params().get('exclusiveTag')

    def set_exclusiveTag(self, exclusiveTag):
        self.add_param('exclusiveTag', exclusiveTag)

    def get_limit(self):
        return self.get_params().get('limit')

    def set_limit(self, limit):
        self.add_param('limit', limit)

    def get_offset(self):
        return self.get_params().get('offset')

    def set_offset(self, offset):
        self.add_param('offset', offset)

    def get_orderBy(self):
        return self.get_params().get('orderBy')

    def set_orderBy(self, orderBy):
        self.add_param('orderBy', orderBy)

    def get_orderType(self):
        return self.get_params().get('orderType')

    def set_orderType(self, orderType):
        self.add_param('orderType', orderType)

    def get_payMode(self):
        return self.get_params().get('payMode')

    def set_payMode(self, payMode):
        self.add_param('payMode', payMode)

    def get_searchKey(self):
        return self.get_params().get('searchKey')

    def set_searchKey(self, searchKey):
        self.add_param('searchKey', searchKey)

    def get_status(self):
        return self.get_params().get('status')

    def set_status(self, status):
        self.add_param('status', status)

    def get_unInstanceIds(self):
        return self.get_params().get('unInstanceIds')

    def set_unInstanceIds(self, unInstanceIds):
        self.add_param('unInstanceIds', unInstanceIds)

    def get_unVpcId(self):
        return self.get_params().get('unVpcId')

    def set_unVpcId(self, unVpcId):
        self.add_param('unVpcId', unVpcId)

    def get_vpcId(self):
        return self.get_params().get('vpcId')

    def set_vpcId(self, vpcId):
        self.add_param('vpcId', vpcId)



# -*- coding: utf-8 -*-

from ..qcloudsdkcore.request import Request

class DescribeBmForwardListenerInfoRequest(Request):

    def __init__(self):
        super(DescribeBmForwardListenerInfoRequest, self).__init__(
            'bmlb', 'qcloudcliV1', 'DescribeBmForwardListenerInfo', 'bmlb.api.qcloud.com')

    def get_ifGetRsInfo(self):
        return self.get_params().get('ifGetRsInfo')

    def set_ifGetRsInfo(self, ifGetRsInfo):
        self.add_param('ifGetRsInfo', ifGetRsInfo)

    def get_instanceIds(self):
        return self.get_params().get('instanceIds')

    def set_instanceIds(self, instanceIds):
        self.add_param('instanceIds', instanceIds)

    def get_loadBalancerId(self):
        return self.get_params().get('loadBalancerId')

    def set_loadBalancerId(self, loadBalancerId):
        self.add_param('loadBalancerId', loadBalancerId)

    def get_searchKey(self):
        return self.get_params().get('searchKey')

    def set_searchKey(self, searchKey):
        self.add_param('searchKey', searchKey)



# -*- coding: utf-8 -*-

from ..qcloudsdkcore.request import Request

class RenewInstanceRequest(Request):

    def __init__(self):
        super(RenewInstanceRequest, self).__init__(
            'cvm', 'qcloudcliV1', 'RenewInstance', 'cvm.api.qcloud.com')

    def get_instanceId(self):
        return self.get_params().get('instanceId')

    def set_instanceId(self, instanceId):
        self.add_param('instanceId', instanceId)

    def get_period(self):
        return self.get_params().get('period')

    def set_period(self, period):
        self.add_param('period', period)

    def get_purchaseSource(self):
        return self.get_params().get('purchaseSource')

    def set_purchaseSource(self, purchaseSource):
        self.add_param('purchaseSource', purchaseSource)



# -*- coding: utf-8 -*-

from ..qcloudsdkcore.request import Request

class ModifyInstanceProjectRequest(Request):

    def __init__(self):
        super(ModifyInstanceProjectRequest, self).__init__(
            'cvm', 'qcloudcliV1', 'ModifyInstanceProject', 'cvm.api.qcloud.com')

    def get_instanceId(self):
        return self.get_params().get('instanceId')

    def set_instanceId(self, instanceId):
        self.add_param('instanceId', instanceId)

    def get_projectId(self):
        return self.get_params().get('projectId')

    def set_projectId(self, projectId):
        self.add_param('projectId', projectId)



# -*- coding: utf-8 -*-

from ..qcloudsdkcore.request import Request

class CreateKeyPairRequest(Request):

    def __init__(self):
        super(CreateKeyPairRequest, self).__init__(
            'cvm', 'qcloudcliV1', 'CreateKeyPair', 'cvm.api.qcloud.com')

    def get_KeyName(self):
        return self.get_params().get('KeyName')

    def set_KeyName(self, KeyName):
        self.add_param('KeyName', KeyName)

    def get_ProjectId(self):
        return self.get_params().get('ProjectId')

    def set_ProjectId(self, ProjectId):
        self.add_param('ProjectId', ProjectId)



# -*- coding: utf-8 -*-

from ..qcloudsdkcore.request import Request

class DescribeBmBindInfoRequest(Request):

    def __init__(self):
        super(DescribeBmBindInfoRequest, self).__init__(
            'bmlb', 'qcloudcliV1', 'DescribeBmBindInfo', 'bmlb.api.qcloud.com')

    def get_instanceIds(self):
        return self.get_params().get('instanceIds')

    def set_instanceIds(self, instanceIds):
        self.add_param('instanceIds', instanceIds)

    def get_unVpcId(self):
        return self.get_params().get('unVpcId')

    def set_unVpcId(self, unVpcId):
        self.add_param('unVpcId', unVpcId)

    def get_vpcId(self):
        return self.get_params().get('vpcId')

    def set_vpcId(self, vpcId):
        self.add_param('vpcId', vpcId)



# coding: utf-8

import logging
import requests

from sa_tools_core.libs.icinga import IcingaClusterConfig

logger = logging.getLogger(__name__)


class DoubanIcingaClusterConfig(IcingaClusterConfig):
    ICINGA_SHIP_TAGS = ('icinga_master', 'icinga_checker')
    NAGBOT_LINK = 'http://nb.douban.com/link/'
    ICINGA_LINK = 'http://icinga.intra.douban.com/icingaweb2/monitoring/service/show?host=${host}&service=${service}'
    SHIP_TAG = 'http://sysadmin.douban.com/ship/search/?pattern={tag}'

    LINK_TYPE_ACK = 1
    LINK_TYPE_REBOOT_HOST = 2

    @classmethod
    def get_link_by_type(cls, env, _type):
        if (env.HOSTSTATE == 'UP'
                or env.SERVICESTATE == 'OK'
                or env.NAGIOS_NOTIFICATIONTYPE == 'ACKNOWLEDGEMENT'):
            return None

        data = {
            'host': env.NAGIOS_HOSTNAME,
            'service': env.NAGIOS_SERVICEDESC,
            'user': env.NAGIOS_CONTACTNAME,
            'ack_type': _type,
            'is_icinga': 1,
        }
        try:
            resp = requests.post(cls.NAGBOT_LINK, data=data)
            return resp.text.strip()
        except Exception:
            logger.exception('get ack link fail')
            return None

    @classmethod
    def get_ack_link(cls, env):
        return cls.get_link_by_type(env, cls.LINK_TYPE_ACK)

    @classmethod
    def get_reboot_host_link(cls, env):
        try:
            if env.TARGET_TYPE == 'service':
                # service
                # if service.ssh.problem: duration >= 10min
                if env.NAGIOS_SERVICEDESC == 'ssh' and float(env.SERVICE_DURATION_SEC) >= 600:
                    return cls.get_link_by_type(env, cls.LINK_TYPE_REBOOT_HOST)
            else:
                # host
                # if host: duration >= 2min
                if env.HOST_DURATION_SEC and float(env.HOST_DURATION_SEC) >= 120:
                    return cls.get_link_by_type(env, cls.LINK_TYPE_REBOOT_HOST)
        except Exception:
            logger.exception('failed to get_reboot_host_link: ')
        return None

    @classmethod
    def get_icinga_link(cls, env):
        return cls.ICINGA_LINK.format(host=env.HOSTALIAS, service=env.SERVICEDESC)

    @classmethod
    def get_icinga_hosts(cls):
        return {'%s.intra.douban.com' % host
                for tag in cls.ICINGA_SHIP_TAGS
                for host in requests.get(cls.SHIP_TAG.format(tag=tag)).json()}



# -*- coding: utf-8 -*-

from ..qcloudsdkcore.request import Request

class DescribeDevicePositionRequest(Request):

    def __init__(self):
        super(DescribeDevicePositionRequest, self).__init__(
            'bm', 'qcloudcliV1', 'DescribeDevicePosition', 'bm.api.qcloud.com')

    def get_alias(self):
        return self.get_params().get('alias')

    def set_alias(self, alias):
        self.add_param('alias', alias)

    def get_instanceIds(self):
        return self.get_params().get('instanceIds')

    def set_instanceIds(self, instanceIds):
        self.add_param('instanceIds', instanceIds)

    def get_limit(self):
        return self.get_params().get('limit')

    def set_limit(self, limit):
        self.add_param('limit', limit)

    def get_offset(self):
        return self.get_params().get('offset')

    def set_offset(self, offset):
        self.add_param('offset', offset)

    def get_subnetId(self):
        return self.get_params().get('subnetId')

    def set_subnetId(self, subnetId):
        self.add_param('subnetId', subnetId)

    def get_vpcId(self):
        return self.get_params().get('vpcId')

    def set_vpcId(self, vpcId):
        self.add_param('vpcId', vpcId)



# -*- coding: utf-8 -*-

from ..qcloudsdkcore.request import Request

class QueryElasticPriceRequest(Request):

    def __init__(self):
        super(QueryElasticPriceRequest, self).__init__(
            'bm', 'qcloudcliV1', 'QueryElasticPrice', 'bm.api.qcloud.com')

    def get_cpuId(self):
        return self.get_params().get('cpuId')

    def set_cpuId(self, cpuId):
        self.add_param('cpuId', cpuId)

    def get_deviceClass(self):
        return self.get_params().get('deviceClass')

    def set_deviceClass(self, deviceClass):
        self.add_param('deviceClass', deviceClass)

    def get_diskNum1(self):
        return self.get_params().get('diskNum1')

    def set_diskNum1(self, diskNum1):
        self.add_param('diskNum1', diskNum1)

    def get_diskNum2(self):
        return self.get_params().get('diskNum2')

    def set_diskNum2(self, diskNum2):
        self.add_param('diskNum2', diskNum2)

    def get_diskTypeId1(self):
        return self.get_params().get('diskTypeId1')

    def set_diskTypeId1(self, diskTypeId1):
        self.add_param('diskTypeId1', diskTypeId1)

    def get_diskTypeId2(self):
        return self.get_params().get('diskTypeId2')

    def set_diskTypeId2(self, diskTypeId2):
        self.add_param('diskTypeId2', diskTypeId2)

    def get_goodsNum(self):
        return self.get_params().get('goodsNum')

    def set_goodsNum(self, goodsNum):
        self.add_param('goodsNum', goodsNum)

    def get_haveRaidCard(self):
        return self.get_params().get('haveRaidCard')

    def set_haveRaidCard(self, haveRaidCard):
        self.add_param('haveRaidCard', haveRaidCard)

    def get_mem(self):
        return self.get_params().get('mem')

    def set_mem(self, mem):
        self.add_param('mem', mem)

    def get_timeSpan(self):
        return self.get_params().get('timeSpan')

    def set_timeSpan(self, timeSpan):
        self.add_param('timeSpan', timeSpan)

    def get_timeUnit(self):
        return self.get_params().get('timeUnit')

    def set_timeUnit(self, timeUnit):
        self.add_param('timeUnit', timeUnit)



# -*- coding: utf-8 -*-

from ..qcloudsdkcore.request import Request

class DescribeInstancesRequest(Request):

    def __init__(self):
        super(DescribeInstancesRequest, self).__init__(
            'cvm', 'qcloudcliV1', 'DescribeInstances', 'cvm.api.qcloud.com')

    def get_instanceIds(self):
        return self.get_params().get('instanceIds')

    def set_instanceIds(self, instanceIds):
        self.add_param('instanceIds', instanceIds)

    def get_lanIps(self):
        return self.get_params().get('lanIps')

    def set_lanIps(self, lanIps):
        self.add_param('lanIps', lanIps)

    def get_limit(self):
        return self.get_params().get('limit')

    def set_limit(self, limit):
        self.add_param('limit', limit)

    def get_offset(self):
        return self.get_params().get('offset')

    def set_offset(self, offset):
        self.add_param('offset', offset)

    def get_projectId(self):
        return self.get_params().get('projectId')

    def set_projectId(self, projectId):
        self.add_param('projectId', projectId)

    def get_searchWord(self):
        return self.get_params().get('searchWord')

    def set_searchWord(self, searchWord):
        self.add_param('searchWord', searchWord)

    def get_simplify(self):
        return self.get_params().get('simplify')

    def set_simplify(self, simplify):
        self.add_param('simplify', simplify)

    def get_status(self):
        return self.get_params().get('status')

    def set_status(self, status):
        self.add_param('status', status)

    def get_zoneId(self):
        return self.get_params().get('zoneId')

    def set_zoneId(self, zoneId):
        self.add_param('zoneId', zoneId)



# -*- coding: utf-8 -*-

from ..qcloudsdkcore.request import Request

class DescribeBmVportInfoRequest(Request):

    def __init__(self):
        super(DescribeBmVportInfoRequest, self).__init__(
            'bmlb', 'qcloudcliV1', 'DescribeBmVportInfo', 'bmlb.api.qcloud.com')

    def get_loadBalancerId(self):
        return self.get_params().get('loadBalancerId')

    def set_loadBalancerId(self, loadBalancerId):
        self.add_param('loadBalancerId', loadBalancerId)



# -*- coding: utf-8 -*-

from ..qcloudsdkcore.request import Request

class DescribeDeviceClassRequest(Request):

    def __init__(self):
        super(DescribeDeviceClassRequest, self).__init__(
            'bm', 'qcloudcliV1', 'DescribeDeviceClass', 'bm.api.qcloud.com')



# -*- coding: utf-8 -*-

from ..qcloudsdkcore.request import Request

class DescribeBmVpcPeersExRequest(Request):

    def __init__(self):
        super(DescribeBmVpcPeersExRequest, self).__init__(
            'bmvpc', 'qcloudcliV1', 'DescribeBmVpcPeersEx', 'bmvpc.api.qcloud.com')

    def get_limit(self):
        return self.get_params().get('limit')

    def set_limit(self, limit):
        self.add_param('limit', limit)

    def get_offset(self):
        return self.get_params().get('offset')

    def set_offset(self, offset):
        self.add_param('offset', offset)

    def get_orderDirection(self):
        return self.get_params().get('orderDirection')

    def set_orderDirection(self, orderDirection):
        self.add_param('orderDirection', orderDirection)

    def get_orderField(self):
        return self.get_params().get('orderField')

    def set_orderField(self, orderField):
        self.add_param('orderField', orderField)

    def get_peeringConnectionIds(self):
        return self.get_params().get('peeringConnectionIds')

    def set_peeringConnectionIds(self, peeringConnectionIds):
        self.add_param('peeringConnectionIds', peeringConnectionIds)

    def get_state(self):
        return self.get_params().get('state')

    def set_state(self, state):
        self.add_param('state', state)



# -*- coding: utf-8 -*-

from ..qcloudsdkcore.request import Request

class InquiryInstancePriceHourRequest(Request):

    def __init__(self):
        super(InquiryInstancePriceHourRequest, self).__init__(
            'cvm', 'qcloudcliV1', 'InquiryInstancePriceHour', 'cvm.api.qcloud.com')

    def get_bandwidth(self):
        return self.get_params().get('bandwidth')

    def set_bandwidth(self, bandwidth):
        self.add_param('bandwidth', bandwidth)

    def get_bandwidthType(self):
        return self.get_params().get('bandwidthType')

    def set_bandwidthType(self, bandwidthType):
        self.add_param('bandwidthType', bandwidthType)

    def get_cpu(self):
        return self.get_params().get('cpu')

    def set_cpu(self, cpu):
        self.add_param('cpu', cpu)

    def get_goodsNum(self):
        return self.get_params().get('goodsNum')

    def set_goodsNum(self, goodsNum):
        self.add_param('goodsNum', goodsNum)

    def get_imageId(self):
        return self.get_params().get('imageId')

    def set_imageId(self, imageId):
        self.add_param('imageId', imageId)

    def get_imageType(self):
        return self.get_params().get('imageType')

    def set_imageType(self, imageType):
        self.add_param('imageType', imageType)

    def get_instanceModel(self):
        return self.get_params().get('instanceModel')

    def set_instanceModel(self, instanceModel):
        self.add_param('instanceModel', instanceModel)

    def get_mem(self):
        return self.get_params().get('mem')

    def set_mem(self, mem):
        self.add_param('mem', mem)

    def get_rootSize(self):
        return self.get_params().get('rootSize')

    def set_rootSize(self, rootSize):
        self.add_param('rootSize', rootSize)

    def get_storageSize(self):
        return self.get_params().get('storageSize')

    def set_storageSize(self, storageSize):
        self.add_param('storageSize', storageSize)

    def get_storageType(self):
        return self.get_params().get('storageType')

    def set_storageType(self, storageType):
        self.add_param('storageType', storageType)

    def get_zoneId(self):
        return self.get_params().get('zoneId')

    def set_zoneId(self, zoneId):
        self.add_param('zoneId', zoneId)



# -*- coding: utf-8 -*-

from ..qcloudsdkcore.request import Request

class ModifyInstanceInternetChargeTypeRequest(Request):

    def __init__(self):
        super(ModifyInstanceInternetChargeTypeRequest, self).__init__(
            'cvm', 'qcloudcliV1', 'ModifyInstanceInternetChargeType', 'cvm.api.qcloud.com')

    def get_DryRun(self):
        return self.get_params().get('DryRun')

    def set_DryRun(self, DryRun):
        self.add_param('DryRun', DryRun)

    def get_InstanceId(self):
        return self.get_params().get('InstanceId')

    def set_InstanceId(self, InstanceId):
        self.add_param('InstanceId', InstanceId)

    def get_InternetAccessible(self):
        return self.get_params().get('InternetAccessible')

    def set_InternetAccessible(self, InternetAccessible):
        self.add_param('InternetAccessible', InternetAccessible)



# -*- coding: utf-8 -*-

from ..qcloudsdkcore.request import Request

class CreateBmSubnetRequest(Request):

    def __init__(self):
        super(CreateBmSubnetRequest, self).__init__(
            'bmvpc', 'qcloudcliV1', 'CreateBmSubnet', 'bmvpc.api.qcloud.com')

    def get_subnetSet(self):
        return self.get_params().get('subnetSet')

    def set_subnetSet(self, subnetSet):
        self.add_param('subnetSet', subnetSet)

    def get_unVpcId(self):
        return self.get_params().get('unVpcId')

    def set_unVpcId(self, unVpcId):
        self.add_param('unVpcId', unVpcId)

    def get_vlanId(self):
        return self.get_params().get('vlanId')

    def set_vlanId(self, vlanId):
        self.add_param('vlanId', vlanId)

    def get_vpcId(self):
        return self.get_params().get('vpcId')

    def set_vpcId(self, vpcId):
        self.add_param('vpcId', vpcId)



# -*- coding: utf-8 -*-

from ..qcloudsdkcore.request import Request

class ResizeInstanceHourRequest(Request):

    def __init__(self):
        super(ResizeInstanceHourRequest, self).__init__(
            'cvm', 'qcloudcliV1', 'ResizeInstanceHour', 'cvm.api.qcloud.com')

    def get_cpu(self):
        return self.get_params().get('cpu')

    def set_cpu(self, cpu):
        self.add_param('cpu', cpu)

    def get_instanceId(self):
        return self.get_params().get('instanceId')

    def set_instanceId(self, instanceId):
        self.add_param('instanceId', instanceId)

    def get_mem(self):
        return self.get_params().get('mem')

    def set_mem(self, mem):
        self.add_param('mem', mem)

    def get_storageSize(self):
        return self.get_params().get('storageSize')

    def set_storageSize(self, storageSize):
        self.add_param('storageSize', storageSize)



# -*- coding: utf-8 -*-

from ..qcloudsdkcore.request import Request

class DescribeEipAclBmRequest(Request):

    def __init__(self):
        super(DescribeEipAclBmRequest, self).__init__(
            'bmeip', 'qcloudcliV1', 'DescribeEipAclBm', 'bmeip.api.qcloud.com')

    def get_aclIds(self):
        return self.get_params().get('aclIds')

    def set_aclIds(self, aclIds):
        self.add_param('aclIds', aclIds)

    def get_aclName(self):
        return self.get_params().get('aclName')

    def set_aclName(self, aclName):
        self.add_param('aclName', aclName)

    def get_limit(self):
        return self.get_params().get('limit')

    def set_limit(self, limit):
        self.add_param('limit', limit)

    def get_offset(self):
        return self.get_params().get('offset')

    def set_offset(self, offset):
        self.add_param('offset', offset)

    def get_order(self):
        return self.get_params().get('order')

    def set_order(self, order):
        self.add_param('order', order)

    def get_orderField(self):
        return self.get_params().get('orderField')

    def set_orderField(self, orderField):
        self.add_param('orderField', orderField)

    def get_query(self):
        return self.get_params().get('query')

    def set_query(self, query):
        self.add_param('query', query)



# coding: utf-8

import os

PROJECT_ROOT = os.path.dirname(os.path.abspath(__file__))

IPMI_TIMEOUT = 2
IPMI_RETRIES = 1

# CONFIGURATION SECTION

CONFIG_DIR = '/etc/sa-tools/'
EXTERNAL_DOMAINS_CONFIG_FILE = CONFIG_DIR + '/external_domains'

NODE_IRC_CHANNEL = '#sysadmin'
NODE_SLACK_CHANNEL = '#sa'
NODE_MAIL_TO = 'sa@example.com'

SYSADMIN_EMAIL = 'sysadmin@example.com'

SMTP_SERVER = ''

PROXIES = {
    "http": "http://gfw:2333",
    "https": "http://gfw:2333",
}

# # Elasticsearch

SA_ES_HOSTS = ['es.svc:8080']
SA_ES_NGINX_ACCESS_INDEX_PREFIX = 'heka-nginx.access-'
SA_ES_NGINX_ACCESS_DOC_TYPE = 'nginx.access'

# # DNS

# can be a string pattern which contains {cb_token}
DNS_MONITOR_CALLBACK_URL = 'http://example.com/api/callback/dnsmonitor/{cb_token}'
DEFAULT_DNS_DOMAIN = 'example.com'

# # Scripts

ENABLE_DOA = False
ANSIBLE_INVENTORY_CONFIG_PATH = ['/etc/ansible/hosts']
ANSIBLE_MODULE_PATH = []

# # Disk

NCDU_EXPORT_DATA_PATH = "/data1"
NCDU_JOB_LOCK_PREFIX = "/tmp/sa-disk-ncdu-lock"

# # Icinga2

ICINGA_EMAIL = 'icinga@example.com'
# see https://docs.sentry.io/error-reporting/configuration/?platform=python for more details
SENTRY_DSN = 'https://<key>@sentry.io/<project>'

# notification gateway is a web service that stores and analyizes notifications. We have one in douban, you can build
# your own.
NOTIFICATION_GATEWAY_API = ''
NOTIFICATION_GATEWAY_TIMEOUT = 10  # 10s

ICINGA_CACERT = '/etc/icinga2/ssl/certs/ca.pem'
# you need to inhert IcingaClusterConfig and impl your own config class
# see sa_tools_core.libs.icinga for more details
ICINGA_CLUSTER_CONFIG_CLASS = 'sa_tools_core.libs.icinga:IcingaClusterConfig'

# # BS(Black Stone)

BS_API_REQUESTS_MODULE_PREFIX = 'sa_tools_core.libs.qcloud.qcloudsdk'
BS_CMD_PATTERN = 'qcloudcli "{module}" "{action}" {params}'
BS_DEFAULT_ATTRS = (
    'alias', 'eipName', 'subnetName',
    # 'vpcName', 'natName', 'instanceAlias',
    # 'instanceId', 'vlanId', 'natUid', 'unInstanceId', 'subnetId',
    'lanIp', 'vpcIp', 'eip',
    'cidrBlock', 'cidr',
)
BS_DEFAULT_PARAMS = {
    'limit': 100,
}
BS_DEFAULT_PARAMS_BM = {
    'vpcId': 1001,
    'unVpcId': 'vpc-xxxxxxxx',
    'zoneId': 1000800001,  # 可用区ID。可通过 DescribeRegions 接口用来获取黑石物理机可用区。
}
BS_PLURAL_SUFFIX = ['s', 'List', 'Set']

try:
    from local_config import *  # NOQA
    exec(open(os.path.join(CONFIG_DIR, "config.py")).read())
except Exception:
    pass



# -*- coding: utf-8 -*-

from ..qcloudsdkcore.request import Request

class DescriptionOperationResultRequest(Request):

    def __init__(self):
        super(DescriptionOperationResultRequest, self).__init__(
            'bm', 'qcloudcliV1', 'DescriptionOperationResult', 'bm.api.qcloud.com')

    def get_taskId(self):
        return self.get_params().get('taskId')

    def set_taskId(self, taskId):
        self.add_param('taskId', taskId)




# -*- coding: utf-8 -*-

from ..qcloudsdkcore.request import Request

class BindBmL4ListenerRsRequest(Request):

    def __init__(self):
        super(BindBmL4ListenerRsRequest, self).__init__(
            'bmlb', 'qcloudcliV1', 'BindBmL4ListenerRs', 'bmlb.api.qcloud.com')

    def get_backends(self):
        return self.get_params().get('backends')

    def set_backends(self, backends):
        self.add_param('backends', backends)

    def get_listenerId(self):
        return self.get_params().get('listenerId')

    def set_listenerId(self, listenerId):
        self.add_param('listenerId', listenerId)

    def get_loadBalancerId(self):
        return self.get_params().get('loadBalancerId')

    def set_loadBalancerId(self, loadBalancerId):
        self.add_param('loadBalancerId', loadBalancerId)



# -*- coding: utf-8 -*-

from ..qcloudsdkcore.request import Request

class InquiryPriceResetInstancesTypeRequest(Request):

    def __init__(self):
        super(InquiryPriceResetInstancesTypeRequest, self).__init__(
            'cvm', 'qcloudcliV1', 'InquiryPriceResetInstancesType', 'cvm.api.qcloud.com')

    def get_DryRun(self):
        return self.get_params().get('DryRun')

    def set_DryRun(self, DryRun):
        self.add_param('DryRun', DryRun)

    def get_ForceStop(self):
        return self.get_params().get('ForceStop')

    def set_ForceStop(self, ForceStop):
        self.add_param('ForceStop', ForceStop)

    def get_InstanceIds(self):
        return self.get_params().get('InstanceIds')

    def set_InstanceIds(self, InstanceIds):
        self.add_param('InstanceIds', InstanceIds)

    def get_InstanceType(self):
        return self.get_params().get('InstanceType')

    def set_InstanceType(self, InstanceType):
        self.add_param('InstanceType', InstanceType)

    def get_Uin(self):
        return self.get_params().get('Uin')

    def set_Uin(self, Uin):
        self.add_param('Uin', Uin)



# -*- coding: utf-8 -*-

from ..qcloudsdkcore.request import Request

class RunInstancesRequest(Request):

    def __init__(self):
        super(RunInstancesRequest, self).__init__(
            'cvm', 'qcloudcliV1', 'RunInstances', 'cvm.api.qcloud.com')

    def get_ClientToken(self):
        return self.get_params().get('ClientToken')

    def set_ClientToken(self, ClientToken):
        self.add_param('ClientToken', ClientToken)

    def get_DataDisks(self):
        return self.get_params().get('DataDisks')

    def set_DataDisks(self, DataDisks):
        self.add_param('DataDisks', DataDisks)

    def get_DryRun(self):
        return self.get_params().get('DryRun')

    def set_DryRun(self, DryRun):
        self.add_param('DryRun', DryRun)

    def get_EnhancedService(self):
        return self.get_params().get('EnhancedService')

    def set_EnhancedService(self, EnhancedService):
        self.add_param('EnhancedService', EnhancedService)

    def get_HostName(self):
        return self.get_params().get('HostName')

    def set_HostName(self, HostName):
        self.add_param('HostName', HostName)

    def get_ImageId(self):
        return self.get_params().get('ImageId')

    def set_ImageId(self, ImageId):
        self.add_param('ImageId', ImageId)

    def get_InstanceChargePrepaid(self):
        return self.get_params().get('InstanceChargePrepaid')

    def set_InstanceChargePrepaid(self, InstanceChargePrepaid):
        self.add_param('InstanceChargePrepaid', InstanceChargePrepaid)

    def get_InstanceChargeType(self):
        return self.get_params().get('InstanceChargeType')

    def set_InstanceChargeType(self, InstanceChargeType):
        self.add_param('InstanceChargeType', InstanceChargeType)

    def get_InstanceCount(self):
        return self.get_params().get('InstanceCount')

    def set_InstanceCount(self, InstanceCount):
        self.add_param('InstanceCount', InstanceCount)

    def get_InstanceName(self):
        return self.get_params().get('InstanceName')

    def set_InstanceName(self, InstanceName):
        self.add_param('InstanceName', InstanceName)

    def get_InstanceType(self):
        return self.get_params().get('InstanceType')

    def set_InstanceType(self, InstanceType):
        self.add_param('InstanceType', InstanceType)

    def get_InternetAccessible(self):
        return self.get_params().get('InternetAccessible')

    def set_InternetAccessible(self, InternetAccessible):
        self.add_param('InternetAccessible', InternetAccessible)

    def get_LoginSettings(self):
        return self.get_params().get('LoginSettings')

    def set_LoginSettings(self, LoginSettings):
        self.add_param('LoginSettings', LoginSettings)

    def get_Placement(self):
        return self.get_params().get('Placement')

    def set_Placement(self, Placement):
        self.add_param('Placement', Placement)

    def get_PurchaseSource(self):
        return self.get_params().get('PurchaseSource')

    def set_PurchaseSource(self, PurchaseSource):
        self.add_param('PurchaseSource', PurchaseSource)

    def get_SecurityGroupIds(self):
        return self.get_params().get('SecurityGroupIds')

    def set_SecurityGroupIds(self, SecurityGroupIds):
        self.add_param('SecurityGroupIds', SecurityGroupIds)

    def get_SystemDisk(self):
        return self.get_params().get('SystemDisk')

    def set_SystemDisk(self, SystemDisk):
        self.add_param('SystemDisk', SystemDisk)

    def get_Uin(self):
        return self.get_params().get('Uin')

    def set_Uin(self, Uin):
        self.add_param('Uin', Uin)

    def get_UserData(self):
        return self.get_params().get('UserData')

    def set_UserData(self, UserData):
        self.add_param('UserData', UserData)

    def get_VirtualPrivateCloud(self):
        return self.get_params().get('VirtualPrivateCloud')

    def set_VirtualPrivateCloud(self, VirtualPrivateCloud):
        self.add_param('VirtualPrivateCloud', VirtualPrivateCloud)



# -*- coding: utf-8 -*-

from ..qcloudsdkcore.request import Request

class TerminateInstancesRequest(Request):

    def __init__(self):
        super(TerminateInstancesRequest, self).__init__(
            'cvm', 'qcloudcliV1', 'TerminateInstances', 'cvm.api.qcloud.com')

    def get_DryRun(self):
        return self.get_params().get('DryRun')

    def set_DryRun(self, DryRun):
        self.add_param('DryRun', DryRun)

    def get_InstanceIds(self):
        return self.get_params().get('InstanceIds')

    def set_InstanceIds(self, InstanceIds):
        self.add_param('InstanceIds', InstanceIds)

    def get_ReleaseAddress(self):
        return self.get_params().get('ReleaseAddress')

    def set_ReleaseAddress(self, ReleaseAddress):
        self.add_param('ReleaseAddress', ReleaseAddress)



# -*- coding: utf-8 -*-

from ..qcloudsdkcore.request import Request

class BuyDeviceRequest(Request):

    def __init__(self):
        super(BuyDeviceRequest, self).__init__(
            'bm', 'qcloudcliV1', 'BuyDevice', 'bm.api.qcloud.com')

    def get_zoneId(self):
        return self.get_params().get('zoneId')

    def set_zoneId(self, zoneId):
        self.add_param('zoneId', zoneId)

    def get_unVpcId(self):
        return self.get_params().get('unVpcId')

    def set_unVpcId(self, unVpcId):
        self.add_param('unVpcId', unVpcId)

    def get_unSubnetId(self):
        return self.get_params().get('unSubnetId')

    def set_unSubnetId(self, unSubnetId):
        self.add_param('unSubnetId', unSubnetId)

    def get_deviceClassCode(self):
        return self.get_params().get('deviceClassCode')

    def set_deviceClassCode(self, deviceClassCode):
        self.add_param('deviceClassCode', deviceClassCode)

    def get_cpuId(self):
        return self.get_params().get('cpuId')

    def set_cpuId(self, cpuId):
        self.add_param('cpuId', cpuId)

    def get_mem(self):
        return self.get_params().get('mem')

    def set_mem(self, mem):
        self.add_param('mem', mem)

    def get_haveRaidCard(self):
        return self.get_params().get('haveRaidCard')

    def set_haveRaidCard(self, haveRaidCard):
        self.add_param('haveRaidCard', haveRaidCard)

    def get_diskTypeId1(self):
        return self.get_params().get('diskTypeId1')

    def set_diskTypeId1(self, diskTypeId1):
        self.add_param('diskTypeId1', diskTypeId1)

    def get_diskNum1(self):
        return self.get_params().get('diskNum1')

    def set_diskNum1(self, diskNum1):
        self.add_param('diskNum1', diskNum1)

    def get_diskTypeId2(self):
        return self.get_params().get('diskTypeId2')

    def set_diskTypeId2(self, diskTypeId2):
        self.add_param('diskTypeId2', diskTypeId2)

    def get_diskNum2(self):
        return self.get_params().get('diskNum2')

    def set_diskNum2(self, diskNum2):
        self.add_param('diskNum2', diskNum2)

    def get_osTypeId(self):
        return self.get_params().get('osTypeId')

    def set_osTypeId(self, osTypeId):
        self.add_param('osTypeId', osTypeId)

    def get_raidId(self):
        return self.get_params().get('raidId')

    def set_raidId(self, raidId):
        self.add_param('raidId', raidId)

    def get_timeUnit(self):
        return self.get_params().get('timeUnit')

    def set_timeUnit(self, timeUnit):
        self.add_param('timeUnit', timeUnit)

    def get_timeSpan(self):
        return self.get_params().get('timeSpan')

    def set_timeSpan(self, timeSpan):
        self.add_param('timeSpan', timeSpan)

    def get_goodsNum(self):
        return self.get_params().get('goodsNum')

    def set_goodsNum(self, goodsNum):
        self.add_param('goodsNum', goodsNum)

    def get_hasWanIp(self):
        return self.get_params().get('hasWanIp')

    def set_hasWanIp(self, hasWanIp):
        self.add_param('hasWanIp', hasWanIp)

    def get_needSecurityAgent(self):
        return self.get_params().get('needSecurityAgent')

    def set_needSecurityAgent(self, needSecurityAgent):
        self.add_param('needSecurityAgent', needSecurityAgent)

    def get_needMonitorAgent(self):
        return self.get_params().get('needMonitorAgent')

    def set_needMonitorAgent(self, needMonitorAgent):
        self.add_param('needMonitorAgent', needMonitorAgent)

    def get_alias(self):
        return self.get_params().get('alias')

    def set_alias(self, alias):
        self.add_param('alias', alias)

    def get_sysRootSpace(self):
        return self.get_params().get('sysRootSpace')

    def set_sysRootSpace(self, sysRootSpace):
        self.add_param('sysRootSpace', sysRootSpace)

    def get_sysSwaporuefiSpace(self):
        return self.get_params().get('sysSwaporuefiSpace')

    def set_sysSwaporuefiSpace(self, sysSwaporuefiSpace):
        self.add_param('sysSwaporuefiSpace', sysSwaporuefiSpace)

    def get_sysDataSpace(self):
        return self.get_params().get('sysDataSpace')

    def set_sysDataSpace(self, sysDataSpace):
        self.add_param('sysDataSpace', sysDataSpace)

    def get_hyperThreading(self):
        return self.get_params().get('hyperThreading')

    def set_hyperThreading(self, hyperThreading):
        self.add_param('hyperThreading', hyperThreading)

    def get_autoRenewFlag(self):
        return self.get_params().get('autoRenewFlag')

    def set_autoRenewFlag(self, autoRenewFlag):
        self.add_param('autoRenewFlag', autoRenewFlag)

    def get_ipList(self):
        return self.get_params().get('ipList')

    def set_ipList(self, ipList):
        self.add_param('ipList', ipList)

    def get_fileSystem(self):
        return self.get_params().get('fileSystem')

    def set_fileSystem(self, fileSystem):
        self.add_param('fileSystem', fileSystem)

    def get_tagDetail(self):
        return self.get_params().get('tagDetail')

    def set_tagDetail(self, tagDetail):
        self.add_param('tagDetail', tagDetail)




from . import (ApplyIpsRequest,CreateBmInterfaceRequest,CreateBmNatGatewayRequest,CreateBmSubnetRequest,DelBmInterfaceRequest,DeleteBmNatGatewayRequest,DeleteBmSubnetRequest,DescribeBmCpmBySubnetIdRequest,DescribeBmNatGatewayRequest,DescribeBmSubnetByCpmIdRequest,DescribeBmSubnetExRequest,DescribeBmSubnetIpsRequest,DescribeBmVpcExRequest,DescribeBmVpcPeersExRequest,EipBindBmNatGatewayRequest,EipUnBindBmNatGatewayRequest,ModifyBmNatGatewayRequest,QueryBmTaskResultRequest,RegisterBatchIpRequest,ReturnIpsRequest,SubnetBindBmNatGatewayRequest,SubnetUnBindBmNatGatewayRequest,UpgradeBmNatGatewayRequest,)



# -*- coding: utf-8 -*-

from ..qcloudsdkcore.request import Request

class DescribeHardwareSpecificationRequest(Request):

    def __init__(self):
        super(DescribeHardwareSpecificationRequest, self).__init__(
            'bm', 'qcloudcliV1', 'DescribeHardwareSpecification', 'bm.api.qcloud.com')




# -*- coding: utf-8 -*-

from ..qcloudsdkcore.request import Request

class CreateBmNatGatewayRequest(Request):

    def __init__(self):
        super(CreateBmNatGatewayRequest, self).__init__(
            'bmvpc', 'qcloudcliV1', 'CreateBmNatGateway', 'bmvpc.api.qcloud.com')

    def get_assignedEipSet(self):
        return self.get_params().get('assignedEipSet')

    def set_assignedEipSet(self, assignedEipSet):
        self.add_param('assignedEipSet', assignedEipSet)

    def get_autoAllocEipNum(self):
        return self.get_params().get('autoAllocEipNum')

    def set_autoAllocEipNum(self, autoAllocEipNum):
        self.add_param('autoAllocEipNum', autoAllocEipNum)

    def get_exclusive(self):
        return self.get_params().get('exclusive')

    def set_exclusive(self, exclusive):
        self.add_param('exclusive', exclusive)

    def get_forwardMode(self):
        return self.get_params().get('forwardMode')

    def set_forwardMode(self, forwardMode):
        self.add_param('forwardMode', forwardMode)

    def get_ips(self):
        return self.get_params().get('ips')

    def set_ips(self, ips):
        self.add_param('ips', ips)

    def get_isBackup(self):
        return self.get_params().get('isBackup')

    def set_isBackup(self, isBackup):
        self.add_param('isBackup', isBackup)

    def get_maxConcurrent(self):
        return self.get_params().get('maxConcurrent')

    def set_maxConcurrent(self, maxConcurrent):
        self.add_param('maxConcurrent', maxConcurrent)

    def get_natName(self):
        return self.get_params().get('natName')

    def set_natName(self, natName):
        self.add_param('natName', natName)

    def get_subnetIds(self):
        return self.get_params().get('subnetIds')

    def set_subnetIds(self, subnetIds):
        self.add_param('subnetIds', subnetIds)

    def get_unSubnetIds(self):
        return self.get_params().get('unSubnetIds')

    def set_unSubnetIds(self, unSubnetIds):
        self.add_param('unSubnetIds', unSubnetIds)

    def get_unVpcId(self):
        return self.get_params().get('unVpcId')

    def set_unVpcId(self, unVpcId):
        self.add_param('unVpcId', unVpcId)

    def get_vpcId(self):
        return self.get_params().get('vpcId')

    def set_vpcId(self, vpcId):
        self.add_param('vpcId', vpcId)



# -*- coding: utf-8 -*-

from ..qcloudsdkcore.request import Request

class EipBmBindRsRequest(Request):

    def __init__(self):
        super(EipBmBindRsRequest, self).__init__(
            'bmeip', 'qcloudcliV1', 'EipBmBindRs', 'bmeip.api.qcloud.com')

    def get_eipId(self):
        return self.get_params().get('eipId')

    def set_eipId(self, eipId):
        self.add_param('eipId', eipId)

    def get_instanceId(self):
        return self.get_params().get('instanceId')

    def set_instanceId(self, instanceId):
        self.add_param('instanceId', instanceId)



# -*- coding: utf-8 -*-

from ..qcloudsdkcore.request import Request

class DescribeDevicePriceRequest(Request):

    def __init__(self):
        super(DescribeDevicePriceRequest, self).__init__(
            'bm', 'qcloudcliV1', 'DescribeDevicePrice', 'bm.api.qcloud.com')

    def get_instanceIds(self):
        return self.get_params().get('instanceIds')

    def set_instanceIds(self, instanceIds):
        self.add_param('instanceIds', instanceIds)

    def get_timeSpan(self):
        return self.get_params().get('timeSpan')

    def set_timeSpan(self, timeSpan):
        self.add_param('timeSpan', timeSpan)

    def get_timeUnit(self):
        return self.get_params().get('timeUnit')

    def set_timeUnit(self, timeUnit):
        self.add_param('timeUnit', timeUnit)



# -*- coding: utf-8 -*-

from ..qcloudsdkcore.request import Request

class QueryBmTaskResultRequest(Request):

    def __init__(self):
        super(QueryBmTaskResultRequest, self).__init__(
            'bmvpc', 'qcloudcliV1', 'QueryBmTaskResult', 'bmvpc.api.qcloud.com')

    def get_taskId(self):
        return self.get_params().get('taskId')

    def set_taskId(self, taskId):
        self.add_param('taskId', taskId)



# -*- coding: utf-8 -*-

from ..qcloudsdkcore.request import Request

class ModifyKeyPairRequest(Request):

    def __init__(self):
        super(ModifyKeyPairRequest, self).__init__(
            'cvm', 'qcloudcliV1', 'ModifyKeyPair', 'cvm.api.qcloud.com')

    def get_keyId(self):
        return self.get_params().get('keyId')

    def set_keyId(self, keyId):
        self.add_param('keyId', keyId)

    def get_keyName(self):
        return self.get_params().get('keyName')

    def set_keyName(self, keyName):
        self.add_param('keyName', keyName)



# coding: utf-8

from __future__ import absolute_import

import sys
import logging
from functools import wraps

import sentry_sdk

from sa_tools_core.consts import SENTRY_DSN

logger = logging.getLogger(__name__)

sentry_sdk.init(SENTRY_DSN)
hub = sentry_sdk.Hub.current


def report(msg=None, **kw):
    try:
        extra = kw.pop('extra', {})
        extra['str(sys.argv)'] = str(sys.argv)

        with sentry_sdk.push_scope() as scope:
            for k, v in extra.items():
                scope.set_extra(k, v)
            scope.level = kw.pop('level', logging.ERROR)

            if 'user' in kw:
                scope.user = kw.get('user')

            if msg:
                # stack = kw.pop('stack', None)
                # if stack is None or stack is True:
                #     kw['stack'] = _gen_stack()

                hub.capture_message(msg, level=scope.level, **kw)
            else:
                exc_info = sys.exc_info()
                hub.capture_exception(error=exc_info, **kw)
    except Exception:
        logger.exception('report to sentry failed: ')


def send_sentry(func):
    @wraps(func)
    def _(*a, **kw):
        try:
            return func(*a, **kw)
        except Exception:
            report()
            raise
    return _


def _gen_stack(limit=30):
    try:
        raise Exception
    except Exception:
        f = sys.exc_info()[2].tb_frame.f_back
    stack = []
    n = 0
    while f and n < limit:
        stack.append(f)
        f = f.f_back
        n += 1
    stack.reverse()
    return stack



# -*- coding: utf-8 -*-

from ..qcloudsdkcore.request import Request

class EipBmApplyRequest(Request):

    def __init__(self):
        super(EipBmApplyRequest, self).__init__(
            'bmeip', 'qcloudcliV1', 'EipBmApply', 'bmeip.api.qcloud.com')

    def get_bandwidth(self):
        return self.get_params().get('bandwidth')

    def set_bandwidth(self, bandwidth):
        self.add_param('bandwidth', bandwidth)

    def get_exclusive(self):
        return self.get_params().get('exclusive')

    def set_exclusive(self, exclusive):
        self.add_param('exclusive', exclusive)

    def get_goodsNum(self):
        return self.get_params().get('goodsNum')

    def set_goodsNum(self, goodsNum):
        self.add_param('goodsNum', goodsNum)

    def get_ipList(self):
        return self.get_params().get('ipList')

    def set_ipList(self, ipList):
        self.add_param('ipList', ipList)

    def get_payMode(self):
        return self.get_params().get('payMode')

    def set_payMode(self, payMode):
        self.add_param('payMode', payMode)

    def get_setType(self):
        return self.get_params().get('setType')

    def set_setType(self, setType):
        self.add_param('setType', setType)

    def get_unVpcId(self):
        return self.get_params().get('unVpcId')

    def set_unVpcId(self, unVpcId):
        self.add_param('unVpcId', unVpcId)

    def get_vpcId(self):
        return self.get_params().get('vpcId')

    def set_vpcId(self, vpcId):
        self.add_param('vpcId', vpcId)



# -*- coding: utf-8 -*-

from ..qcloudsdkcore.request import Request

class DescribeInstancesRequest(Request):

    def __init__(self):
        super(DescribeInstancesRequest, self).__init__(
            'cvm', 'qcloudcliV1', 'DescribeInstances', 'cvm.api.qcloud.com')

    def get_Filters(self):
        return self.get_params().get('Filters')

    def set_Filters(self, Filters):
        self.add_param('Filters', Filters)

    def get_InnerSubnetIds(self):
        return self.get_params().get('InnerSubnetIds')

    def set_InnerSubnetIds(self, InnerSubnetIds):
        self.add_param('InnerSubnetIds', InnerSubnetIds)

    def get_InnerVpcIds(self):
        return self.get_params().get('InnerVpcIds')

    def set_InnerVpcIds(self, InnerVpcIds):
        self.add_param('InnerVpcIds', InnerVpcIds)

    def get_InstanceIds(self):
        return self.get_params().get('InstanceIds')

    def set_InstanceIds(self, InstanceIds):
        self.add_param('InstanceIds', InstanceIds)

    def get_IpAddresses(self):
        return self.get_params().get('IpAddresses')

    def set_IpAddresses(self, IpAddresses):
        self.add_param('IpAddresses', IpAddresses)

    def get_Limit(self):
        return self.get_params().get('Limit')

    def set_Limit(self, Limit):
        self.add_param('Limit', Limit)

    def get_Offset(self):
        return self.get_params().get('Offset')

    def set_Offset(self, Offset):
        self.add_param('Offset', Offset)

    def get_VagueInstanceName(self):
        return self.get_params().get('VagueInstanceName')

    def set_VagueInstanceName(self, VagueInstanceName):
        self.add_param('VagueInstanceName', VagueInstanceName)

    def get_VagueIpAddress(self):
        return self.get_params().get('VagueIpAddress')

    def set_VagueIpAddress(self, VagueIpAddress):
        self.add_param('VagueIpAddress', VagueIpAddress)



# -*- coding: utf-8 -*-

from ..qcloudsdkcore.request import Request

class DescribeInstanceTypeConfigsRequest(Request):

    def __init__(self):
        super(DescribeInstanceTypeConfigsRequest, self).__init__(
            'cvm', 'qcloudcliV1', 'DescribeInstanceTypeConfigs', 'cvm.api.qcloud.com')

    def get_Filters(self):
        return self.get_params().get('Filters')

    def set_Filters(self, Filters):
        self.add_param('Filters', Filters)



from . import (BindBmL4ListenerRsRequest,BindBmL4ListenerVmIpRequest,BindBmL7LocationVmIpRequest,BindBmLocationInstancesRequest,CreateBmForwardListenersRequest,CreateBmForwardRulesRequest,CreateBmListenersRequest,CreateBmLoadBalancerRequest,DeleteBmForwardDomainsRequest,DeleteBmForwardRulesRequest,DeleteBmListenersRequest,DeleteBmLoadBalancersRequest,DescribeBmBindInfoRequest,DescribeBmForwardListenerInfoRequest,DescribeBmForwardListenersRequest,DescribeBmForwardRulesRequest,DescribeBmL4ListenerBackendsRequest,DescribeBmListenerInfoRequest,DescribeBmListenersRequest,DescribeBmLoadBalancersRequest,DescribeBmLoadBalancersTaskResultRequest,DescribeBmLocationBackendsRequest,DescribeBmVportInfoRequest,InquiryBmLBPriceRequest,ModifyBmForwardListenerRequest,ModifyBmForwardLocationRequest,ModifyBmL4ListenerBackendPortRequest,ModifyBmL4ListenerBackendProbePortRequest,ModifyBmL4ListenerBackendWeightRequest,ModifyBmListenerRequest,ModifyBmLoadBalancerAttributesRequest,ModifyBmLocationBackendPortRequest,ModifyBmLocationBackendWeightRequest,UnbindBmL4ListenerRsRequest,UnbindBmL4ListenerVmIpRequest,UnbindBmL7LocationVmIpRequest,UnbindBmLocationInstancesRequest,)



# -*- coding: utf-8 -*-

from ..qcloudsdkcore.request import Request

class DescribeBmSubnetExRequest(Request):

    def __init__(self):
        super(DescribeBmSubnetExRequest, self).__init__(
            'bmvpc', 'qcloudcliV1', 'DescribeBmSubnetEx', 'bmvpc.api.qcloud.com')

    def get_limit(self):
        return self.get_params().get('limit')

    def set_limit(self, limit):
        self.add_param('limit', limit)

    def get_offset(self):
        return self.get_params().get('offset')

    def set_offset(self, offset):
        self.add_param('offset', offset)

    def get_orderDirection(self):
        return self.get_params().get('orderDirection')

    def set_orderDirection(self, orderDirection):
        self.add_param('orderDirection', orderDirection)

    def get_orderFeild(self):
        return self.get_params().get('orderFeild')

    def set_orderFeild(self, orderFeild):
        self.add_param('orderFeild', orderFeild)

    def get_subnetId(self):
        return self.get_params().get('subnetId')

    def set_subnetId(self, subnetId):
        self.add_param('subnetId', subnetId)

    def get_subnetName(self):
        return self.get_params().get('subnetName')

    def set_subnetName(self, subnetName):
        self.add_param('subnetName', subnetName)

    def get_type(self):
        return self.get_params().get('type')

    def set_type(self, type):
        self.add_param('type', type)

    def get_unSubnetId(self):
        return self.get_params().get('unSubnetId')

    def set_unSubnetId(self, unSubnetId):
        self.add_param('unSubnetId', unSubnetId)

    def get_unVpcId(self):
        return self.get_params().get('unVpcId')

    def set_unVpcId(self, unVpcId):
        self.add_param('unVpcId', unVpcId)

    def get_vlanId(self):
        return self.get_params().get('vlanId')

    def set_vlanId(self, vlanId):
        self.add_param('vlanId', vlanId)

    def get_vpcId(self):
        return self.get_params().get('vpcId')

    def set_vpcId(self, vpcId):
        self.add_param('vpcId', vpcId)

    def get_zoneIds(self):
        return self.get_params().get('zoneIds')

    def set_zoneIds(self, zoneIds):
        self.add_param('zoneIds', zoneIds)



# -*- coding: utf-8 -*-

from ..qcloudsdkcore.request import Request

class DescribeDeviceRequest(Request):

    def __init__(self):
        super(DescribeDeviceRequest, self).__init__(
            'bm', 'qcloudcliV1', 'DescribeDevice', 'bm.api.qcloud.com')

    def get_alias(self):
        return self.get_params().get('alias')

    def set_alias(self, alias):
        self.add_param('alias', alias)

    def get_autoRenewFlag(self):
        return self.get_params().get('autoRenewFlag')

    def set_autoRenewFlag(self, autoRenewFlag):
        self.add_param('autoRenewFlag', autoRenewFlag)

    def get_deadlineEndTime(self):
        return self.get_params().get('deadlineEndTime')

    def set_deadlineEndTime(self, deadlineEndTime):
        self.add_param('deadlineEndTime', deadlineEndTime)

    def get_deadlineStartTime(self):
        return self.get_params().get('deadlineStartTime')

    def set_deadlineStartTime(self, deadlineStartTime):
        self.add_param('deadlineStartTime', deadlineStartTime)

    def get_deviceClassCode(self):
        return self.get_params().get('deviceClassCode')

    def set_deviceClassCode(self, deviceClassCode):
        self.add_param('deviceClassCode', deviceClassCode)

    def get_instanceIds(self):
        return self.get_params().get('instanceIds')

    def set_instanceIds(self, instanceIds):
        self.add_param('instanceIds', instanceIds)

    def get_lanIps(self):
        return self.get_params().get('lanIps')

    def set_lanIps(self, lanIps):
        self.add_param('lanIps', lanIps)

    def get_limit(self):
        return self.get_params().get('limit')

    def set_limit(self, limit):
        self.add_param('limit', limit)

    def get_mgtIps(self):
        return self.get_params().get('mgtIps')

    def set_mgtIps(self, mgtIps):
        self.add_param('mgtIps', mgtIps)

    def get_offset(self):
        return self.get_params().get('offset')

    def set_offset(self, offset):
        self.add_param('offset', offset)

    def get_order(self):
        return self.get_params().get('order')

    def set_order(self, order):
        self.add_param('order', order)

    def get_orderField(self):
        return self.get_params().get('orderField')

    def set_orderField(self, orderField):
        self.add_param('orderField', orderField)

    def get_projectId(self):
        return self.get_params().get('projectId')

    def set_projectId(self, projectId):
        self.add_param('projectId', projectId)

    def get_status(self):
        return self.get_params().get('status')

    def set_status(self, status):
        self.add_param('status', status)

    def get_subnetId(self):
        return self.get_params().get('subnetId')

    def set_subnetId(self, subnetId):
        self.add_param('subnetId', subnetId)

    def get_subnetIds(self):
        return self.get_params().get('subnetIds')

    def set_subnetIds(self, subnetIds):
        self.add_param('subnetIds', subnetIds)

    def get_unSubnetIds(self):
        return self.get_params().get('unSubnetIds')

    def set_unSubnetIds(self, unSubnetIds):
        self.add_param('unSubnetIds', unSubnetIds)

    def get_unVpcId(self):
        return self.get_params().get('unVpcId')

    def set_unVpcId(self, unVpcId):
        self.add_param('unVpcId', unVpcId)

    def get_vagueIp(self):
        return self.get_params().get('vagueIp')

    def set_vagueIp(self, vagueIp):
        self.add_param('vagueIp', vagueIp)

    def get_vpcId(self):
        return self.get_params().get('vpcId')

    def set_vpcId(self, vpcId):
        self.add_param('vpcId', vpcId)

    def get_wanIps(self):
        return self.get_params().get('wanIps')

    def set_wanIps(self, wanIps):
        self.add_param('wanIps', wanIps)



# -*- coding: utf-8 -*-

from ..qcloudsdkcore.request import Request

class ResetInstancesRequest(Request):

    def __init__(self):
        super(ResetInstancesRequest, self).__init__(
            'cvm', 'qcloudcliV1', 'ResetInstances', 'cvm.api.qcloud.com')

    def get_hardPowerOffFlag(self):
        return self.get_params().get('hardPowerOffFlag')

    def set_hardPowerOffFlag(self, hardPowerOffFlag):
        self.add_param('hardPowerOffFlag', hardPowerOffFlag)

    def get_imageId(self):
        return self.get_params().get('imageId')

    def set_imageId(self, imageId):
        self.add_param('imageId', imageId)

    def get_imageType(self):
        return self.get_params().get('imageType')

    def set_imageType(self, imageType):
        self.add_param('imageType', imageType)

    def get_instanceId(self):
        return self.get_params().get('instanceId')

    def set_instanceId(self, instanceId):
        self.add_param('instanceId', instanceId)

    def get_needMonitorAgent(self):
        return self.get_params().get('needMonitorAgent')

    def set_needMonitorAgent(self, needMonitorAgent):
        self.add_param('needMonitorAgent', needMonitorAgent)

    def get_needSecurityAgent(self):
        return self.get_params().get('needSecurityAgent')

    def set_needSecurityAgent(self, needSecurityAgent):
        self.add_param('needSecurityAgent', needSecurityAgent)

    def get_password(self):
        return self.get_params().get('password')

    def set_password(self, password):
        self.add_param('password', password)

    def get_purchaseSource(self):
        return self.get_params().get('purchaseSource')

    def set_purchaseSource(self, purchaseSource):
        self.add_param('purchaseSource', purchaseSource)

    def get_rootSize(self):
        return self.get_params().get('rootSize')

    def set_rootSize(self, rootSize):
        self.add_param('rootSize', rootSize)



# -*- coding: utf-8 -*-

from ..qcloudsdkcore.request import Request

class DescribeBmLoadBalancersRequest(Request):

    def __init__(self):
        super(DescribeBmLoadBalancersRequest, self).__init__(
            'bmlb', 'qcloudcliV1', 'DescribeBmLoadBalancers', 'bmlb.api.qcloud.com')

    def get_domain(self):
        return self.get_params().get('domain')

    def set_domain(self, domain):
        self.add_param('domain', domain)

    def get_exclusive(self):
        return self.get_params().get('exclusive')

    def set_exclusive(self, exclusive):
        self.add_param('exclusive', exclusive)

    def get_limit(self):
        return self.get_params().get('limit')

    def set_limit(self, limit):
        self.add_param('limit', limit)

    def get_loadBalancerIds(self):
        return self.get_params().get('loadBalancerIds')

    def set_loadBalancerIds(self, loadBalancerIds):
        self.add_param('loadBalancerIds', loadBalancerIds)

    def get_loadBalancerName(self):
        return self.get_params().get('loadBalancerName')

    def set_loadBalancerName(self, loadBalancerName):
        self.add_param('loadBalancerName', loadBalancerName)

    def get_loadBalancerType(self):
        return self.get_params().get('loadBalancerType')

    def set_loadBalancerType(self, loadBalancerType):
        self.add_param('loadBalancerType', loadBalancerType)

    def get_loadBalancerVips(self):
        return self.get_params().get('loadBalancerVips')

    def set_loadBalancerVips(self, loadBalancerVips):
        self.add_param('loadBalancerVips', loadBalancerVips)

    def get_offset(self):
        return self.get_params().get('offset')

    def set_offset(self, offset):
        self.add_param('offset', offset)

    def get_orderBy(self):
        return self.get_params().get('orderBy')

    def set_orderBy(self, orderBy):
        self.add_param('orderBy', orderBy)

    def get_orderType(self):
        return self.get_params().get('orderType')

    def set_orderType(self, orderType):
        self.add_param('orderType', orderType)

    def get_projectId(self):
        return self.get_params().get('projectId')

    def set_projectId(self, projectId):
        self.add_param('projectId', projectId)

    def get_searchKey(self):
        return self.get_params().get('searchKey')

    def set_searchKey(self, searchKey):
        self.add_param('searchKey', searchKey)

    def get_tgwSetType(self):
        return self.get_params().get('tgwSetType')

    def set_tgwSetType(self, tgwSetType):
        self.add_param('tgwSetType', tgwSetType)

    def get_unVpcId(self):
        return self.get_params().get('unVpcId')

    def set_unVpcId(self, unVpcId):
        self.add_param('unVpcId', unVpcId)



# -*- coding: utf-8 -*-

from ..qcloudsdkcore.request import Request

class ReloadDeviceOsRequest(Request):

    def __init__(self):
        super(ReloadDeviceOsRequest, self).__init__(
            'bm', 'qcloudcliV1', 'ReloadDeviceOs', 'bm.api.qcloud.com')

    def get_agentIds(self):
        return self.get_params().get('agentIds')

    def set_agentIds(self, agentIds):
        self.add_param('agentIds', agentIds)

    def get_hyperThreading(self):
        return self.get_params().get('hyperThreading')

    def set_hyperThreading(self, hyperThreading):
        self.add_param('hyperThreading', hyperThreading)

    def get_instanceId(self):
        return self.get_params().get('instanceId')

    def set_instanceId(self, instanceId):
        self.add_param('instanceId', instanceId)

    def get_isZoning(self):
        return self.get_params().get('isZoning')

    def set_isZoning(self, isZoning):
        self.add_param('isZoning', isZoning)

    def get_lanIp(self):
        return self.get_params().get('lanIp')

    def set_lanIp(self, lanIp):
        self.add_param('lanIp', lanIp)

    def get_osTypeId(self):
        return self.get_params().get('osTypeId')

    def set_osTypeId(self, osTypeId):
        self.add_param('osTypeId', osTypeId)

    def get_passwd(self):
        return self.get_params().get('passwd')

    def set_passwd(self, passwd):
        self.add_param('passwd', passwd)

    def get_raidId(self):
        return self.get_params().get('raidId')

    def set_raidId(self, raidId):
        self.add_param('raidId', raidId)

    def get_subnetId(self):
        return self.get_params().get('subnetId')

    def set_subnetId(self, subnetId):
        self.add_param('subnetId', subnetId)

    def get_sysDataSpace(self):
        return self.get_params().get('sysDataSpace')

    def set_sysDataSpace(self, sysDataSpace):
        self.add_param('sysDataSpace', sysDataSpace)

    def get_sysRootSpace(self):
        return self.get_params().get('sysRootSpace')

    def set_sysRootSpace(self, sysRootSpace):
        self.add_param('sysRootSpace', sysRootSpace)

    def get_sysSwaporuefiSpace(self):
        return self.get_params().get('sysSwaporuefiSpace')

    def set_sysSwaporuefiSpace(self, sysSwaporuefiSpace):
        self.add_param('sysSwaporuefiSpace', sysSwaporuefiSpace)

    def get_sysUsrlocalSpace(self):
        return self.get_params().get('sysUsrlocalSpace')

    def set_sysUsrlocalSpace(self, sysUsrlocalSpace):
        self.add_param('sysUsrlocalSpace', sysUsrlocalSpace)

    def get_unSubnetId(self):
        return self.get_params().get('unSubnetId')

    def set_unSubnetId(self, unSubnetId):
        self.add_param('unSubnetId', unSubnetId)

    def get_unVpcId(self):
        return self.get_params().get('unVpcId')

    def set_unVpcId(self, unVpcId):
        self.add_param('unVpcId', unVpcId)

    def get_vpcId(self):
        return self.get_params().get('vpcId')

    def set_vpcId(self, vpcId):
        self.add_param('vpcId', vpcId)



# -*- coding: utf-8 -*-

from ..qcloudsdkcore.request import Request

class ModifyBmForwardLocationRequest(Request):

    def __init__(self):
        super(ModifyBmForwardLocationRequest, self).__init__(
            'bmlb', 'qcloudcliV1', 'ModifyBmForwardLocation', 'bmlb.api.qcloud.com')

    def get_listenerId(self):
        return self.get_params().get('listenerId')

    def set_listenerId(self, listenerId):
        self.add_param('listenerId', listenerId)

    def get_loadBalancerId(self):
        return self.get_params().get('loadBalancerId')

    def set_loadBalancerId(self, loadBalancerId):
        self.add_param('loadBalancerId', loadBalancerId)

    def get_rules(self):
        return self.get_params().get('rules')

    def set_rules(self, rules):
        self.add_param('rules', rules)



# -*- coding: utf-8 -*-

from ..qcloudsdkcore.request import Request

class ModifyInstancesAttributeRequest(Request):

    def __init__(self):
        super(ModifyInstancesAttributeRequest, self).__init__(
            'cvm', 'qcloudcliV1', 'ModifyInstancesAttribute', 'cvm.api.qcloud.com')

    def get_InstanceIds(self):
        return self.get_params().get('InstanceIds')

    def set_InstanceIds(self, InstanceIds):
        self.add_param('InstanceIds', InstanceIds)

    def get_InstanceName(self):
        return self.get_params().get('InstanceName')

    def set_InstanceName(self, InstanceName):
        self.add_param('InstanceName', InstanceName)

    def get_ResetNewCreationIdentify(self):
        return self.get_params().get('ResetNewCreationIdentify')

    def set_ResetNewCreationIdentify(self, ResetNewCreationIdentify):
        self.add_param('ResetNewCreationIdentify', ResetNewCreationIdentify)

    def get_SecurityGroups(self):
        return self.get_params().get('SecurityGroups')

    def set_SecurityGroups(self, SecurityGroups):
        self.add_param('SecurityGroups', SecurityGroups)

    def get_UserData(self):
        return self.get_params().get('UserData')

    def set_UserData(self, UserData):
        self.add_param('UserData', UserData)



# -*- coding: utf-8 -*-

from ..qcloudsdkcore.request import Request

class UnbindBmL7LocationVmIpRequest(Request):

    def __init__(self):
        super(UnbindBmL7LocationVmIpRequest, self).__init__(
            'bmlb', 'qcloudcliV1', 'UnbindBmL7LocationVmIp', 'bmlb.api.qcloud.com')

    def get_domainId(self):
        return self.get_params().get('domainId')

    def set_domainId(self, domainId):
        self.add_param('domainId', domainId)

    def get_listenerId(self):
        return self.get_params().get('listenerId')

    def set_listenerId(self, listenerId):
        self.add_param('listenerId', listenerId)

    def get_loadBalancerId(self):
        return self.get_params().get('loadBalancerId')

    def set_loadBalancerId(self, loadBalancerId):
        self.add_param('loadBalancerId', loadBalancerId)

    def get_locationId(self):
        return self.get_params().get('locationId')

    def set_locationId(self, locationId):
        self.add_param('locationId', locationId)

    def get_vmList(self):
        return self.get_params().get('vmList')

    def set_vmList(self, vmList):
        self.add_param('vmList', vmList)



# -*- coding: utf-8 -*-

from ..qcloudsdkcore.request import Request

class DescribeAvailabilityZonesRequest(Request):

    def __init__(self):
        super(DescribeAvailabilityZonesRequest, self).__init__(
            'cvm', 'qcloudcliV1', 'DescribeAvailabilityZones', 'cvm.api.qcloud.com')

    def get_zoneId(self):
        return self.get_params().get('zoneId')

    def set_zoneId(self, zoneId):
        self.add_param('zoneId', zoneId)



# -*- coding: utf-8 -*-

from ..qcloudsdkcore.request import Request

class ModifyEipAliasRequest(Request):

    def __init__(self):
        super(ModifyEipAliasRequest, self).__init__(
            'bmeip', 'qcloudcliV1', 'ModifyEipAlias', 'bmeip.api.qcloud.com')

    def get_eipId(self):
        return self.get_params().get('eipId')

    def set_eipId(self, eipId):
        self.add_param('eipId', eipId)

    def get_eipName(self):
        return self.get_params().get('eipName')

    def set_eipName(self, eipName):
        self.add_param('eipName', eipName)



# -*- coding: utf-8 -*-

from ..qcloudsdkcore.request import Request

class EipBmUnBindVpcIpRequest(Request):

    def __init__(self):
        super(EipBmUnBindVpcIpRequest, self).__init__(
            'bmeip', 'qcloudcliV1', 'EipBmUnBindVpcIp', 'bmeip.api.qcloud.com')

    def get_eipId(self):
        return self.get_params().get('eipId')

    def set_eipId(self, eipId):
        self.add_param('eipId', eipId)

    def get_unVpcId(self):
        return self.get_params().get('unVpcId')

    def set_unVpcId(self, unVpcId):
        self.add_param('unVpcId', unVpcId)

    def get_vpcId(self):
        return self.get_params().get('vpcId')

    def set_vpcId(self, vpcId):
        self.add_param('vpcId', vpcId)

    def get_vpcIp(self):
        return self.get_params().get('vpcIp')

    def set_vpcIp(self, vpcIp):
        self.add_param('vpcIp', vpcIp)



# -*- coding: utf-8 -*-

from ..qcloudsdkcore.request import Request

class BindBmL4ListenerVmIpRequest(Request):

    def __init__(self):
        super(BindBmL4ListenerVmIpRequest, self).__init__(
            'bmlb', 'qcloudcliV1', 'BindBmL4ListenerVmIp', 'bmlb.api.qcloud.com')

    def get_listenerId(self):
        return self.get_params().get('listenerId')

    def set_listenerId(self, listenerId):
        self.add_param('listenerId', listenerId)

    def get_loadBalancerId(self):
        return self.get_params().get('loadBalancerId')

    def set_loadBalancerId(self, loadBalancerId):
        self.add_param('loadBalancerId', loadBalancerId)

    def get_vmList(self):
        return self.get_params().get('vmList')

    def set_vmList(self, vmList):
        self.add_param('vmList', vmList)



# -*- coding: utf-8 -*-

from ..qcloudsdkcore.request import Request

class EipAclBmUnBindRequest(Request):

    def __init__(self):
        super(EipAclBmUnBindRequest, self).__init__(
            'bmeip', 'qcloudcliV1', 'EipAclBmUnBind', 'bmeip.api.qcloud.com')

    def get_eipAcl(self):
        return self.get_params().get('eipAcl')

    def set_eipAcl(self, eipAcl):
        self.add_param('eipAcl', eipAcl)



# -*- coding: utf-8 -*-

from ..qcloudsdkcore.request import Request

class DescribeKeyPairsRequest(Request):

    def __init__(self):
        super(DescribeKeyPairsRequest, self).__init__(
            'cvm', 'qcloudcliV1', 'DescribeKeyPairs', 'cvm.api.qcloud.com')

    def get_keyIds(self):
        return self.get_params().get('keyIds')

    def set_keyIds(self, keyIds):
        self.add_param('keyIds', keyIds)

    def get_keyName(self):
        return self.get_params().get('keyName')

    def set_keyName(self, keyName):
        self.add_param('keyName', keyName)

    def get_limit(self):
        return self.get_params().get('limit')

    def set_limit(self, limit):
        self.add_param('limit', limit)

    def get_offset(self):
        return self.get_params().get('offset')

    def set_offset(self, offset):
        self.add_param('offset', offset)

    def get_projectId(self):
        return self.get_params().get('projectId')

    def set_projectId(self, projectId):
        self.add_param('projectId', projectId)



# -*- coding: utf-8 -*-

from ..qcloudsdkcore.request import Request

class InquiryBmLBPriceRequest(Request):

    def __init__(self):
        super(InquiryBmLBPriceRequest, self).__init__(
            'bmlb', 'qcloudcliV1', 'InquiryBmLBPrice', 'bmlb.api.qcloud.com')

    def get_loadBalancerType(self):
        return self.get_params().get('loadBalancerType')

    def set_loadBalancerType(self, loadBalancerType):
        self.add_param('loadBalancerType', loadBalancerType)



# -*- coding: utf-8 -*-

from ..qcloudsdkcore.request import Request

class EipAclBmDeleteRequest(Request):

    def __init__(self):
        super(EipAclBmDeleteRequest, self).__init__(
            'bmeip', 'qcloudcliV1', 'EipAclBmDelete', 'bmeip.api.qcloud.com')

    def get_aclId(self):
        return self.get_params().get('aclId')

    def set_aclId(self, aclId):
        self.add_param('aclId', aclId)



# -*- coding: utf-8 -*-

from ..qcloudsdkcore.request import Request

class EipAclBmSetRequest(Request):

    def __init__(self):
        super(EipAclBmSetRequest, self).__init__(
            'bmeip', 'qcloudcliV1', 'EipAclBmSet', 'bmeip.api.qcloud.com')

    def get_aclId(self):
        return self.get_params().get('aclId')

    def set_aclId(self, aclId):
        self.add_param('aclId', aclId)

    def get_aclName(self):
        return self.get_params().get('aclName')

    def set_aclName(self, aclName):
        self.add_param('aclName', aclName)

    def get_rule(self):
        return self.get_params().get('rule')

    def set_rule(self, rule):
        self.add_param('rule', rule)

    def get_status(self):
        return self.get_params().get('status')

    def set_status(self, status):
        self.add_param('status', status)

    def get_type(self):
        return self.get_params().get('type')

    def set_type(self, type):
        self.add_param('type', type)



from . import (DescribeEipAclBmRequest,DescribeEipBmQuotaRequest,DescribeEipBmRequest,EipAclBmApplyRequest,EipAclBmDeleteRequest,EipAclBmSetRequest,EipAclBmUnBindRequest,EipBmApplyRequest,EipBmBindRsRequest,EipBmBindVpcIpRequest,EipBmDeleteRequest,EipBmModifyChargeRequest,EipBmQueryTaskRequest,EipBmUnbindRsRequest,EipBmUnBindVpcIpRequest,ModifyEipAliasRequest,)



# -*- coding: utf-8 -*-

from ..qcloudsdkcore.request import Request

class DescribeBmNatGatewayRequest(Request):

    def __init__(self):
        super(DescribeBmNatGatewayRequest, self).__init__(
            'bmvpc', 'qcloudcliV1', 'DescribeBmNatGateway', 'bmvpc.api.qcloud.com')

    def get_limit(self):
        return self.get_params().get('limit')

    def set_limit(self, limit):
        self.add_param('limit', limit)

    def get_natId(self):
        return self.get_params().get('natId')

    def set_natId(self, natId):
        self.add_param('natId', natId)

    def get_natName(self):
        return self.get_params().get('natName')

    def set_natName(self, natName):
        self.add_param('natName', natName)

    def get_offset(self):
        return self.get_params().get('offset')

    def set_offset(self, offset):
        self.add_param('offset', offset)

    def get_orderDirection(self):
        return self.get_params().get('orderDirection')

    def set_orderDirection(self, orderDirection):
        self.add_param('orderDirection', orderDirection)

    def get_orderFeild(self):
        return self.get_params().get('orderFeild')

    def set_orderFeild(self, orderFeild):
        self.add_param('orderFeild', orderFeild)

    def get_unVpcId(self):
        return self.get_params().get('unVpcId')

    def set_unVpcId(self, unVpcId):
        self.add_param('unVpcId', unVpcId)

    def get_vpcId(self):
        return self.get_params().get('vpcId')

    def set_vpcId(self, vpcId):
        self.add_param('vpcId', vpcId)



# -*- coding: utf-8 -*-

from ..qcloudsdkcore.request import Request

class GetDeviceDeployProcessRequest(Request):

    def __init__(self):
        super(GetDeviceDeployProcessRequest, self).__init__(
            'bm', 'qcloudcliV1', 'GetDeviceDeployProcess', 'bm.api.qcloud.com')

    def get_instanceId(self):
        return self.get_params().get('instanceId')

    def set_instanceId(self, instanceId):
        self.add_param('instanceId', instanceId)

    def get_isElastic(self):
        return self.get_params().get('isElastic')

    def set_isElastic(self, isElastic):
        self.add_param('isElastic', isElastic)



# -*- coding: utf-8 -*-

from ..qcloudsdkcore.request import Request

class DescribeInstancesStatusRequest(Request):

    def __init__(self):
        super(DescribeInstancesStatusRequest, self).__init__(
            'cvm', 'qcloudcliV1', 'DescribeInstancesStatus', 'cvm.api.qcloud.com')

    def get_InstanceIds(self):
        return self.get_params().get('InstanceIds')

    def set_InstanceIds(self, InstanceIds):
        self.add_param('InstanceIds', InstanceIds)

    def get_Limit(self):
        return self.get_params().get('Limit')

    def set_Limit(self, Limit):
        self.add_param('Limit', Limit)

    def get_Offset(self):
        return self.get_params().get('Offset')

    def set_Offset(self, Offset):
        self.add_param('Offset', Offset)



# -*- coding: utf-8 -*-

from ..qcloudsdkcore.request import Request

class ModifyBmL4ListenerBackendWeightRequest(Request):

    def __init__(self):
        super(ModifyBmL4ListenerBackendWeightRequest, self).__init__(
            'bmlb', 'qcloudcliV1', 'ModifyBmL4ListenerBackendWeight', 'bmlb.api.qcloud.com')

    def get_instanceId(self):
        return self.get_params().get('instanceId')

    def set_instanceId(self, instanceId):
        self.add_param('instanceId', instanceId)

    def get_listenerId(self):
        return self.get_params().get('listenerId')

    def set_listenerId(self, listenerId):
        self.add_param('listenerId', listenerId)

    def get_loadBalancerId(self):
        return self.get_params().get('loadBalancerId')

    def set_loadBalancerId(self, loadBalancerId):
        self.add_param('loadBalancerId', loadBalancerId)

    def get_port(self):
        return self.get_params().get('port')

    def set_port(self, port):
        self.add_param('port', port)

    def get_weight(self):
        return self.get_params().get('weight')

    def set_weight(self, weight):
        self.add_param('weight', weight)



# -*- coding: utf-8 -*-

from ..qcloudsdkcore.request import Request

class ModifyBmForwardListenerRequest(Request):

    def __init__(self):
        super(ModifyBmForwardListenerRequest, self).__init__(
            'bmlb', 'qcloudcliV1', 'ModifyBmForwardListener', 'bmlb.api.qcloud.com')

    def get_bandwidth(self):
        return self.get_params().get('bandwidth')

    def set_bandwidth(self, bandwidth):
        self.add_param('bandwidth', bandwidth)

    def get_certCaContent(self):
        return self.get_params().get('certCaContent')

    def set_certCaContent(self, certCaContent):
        self.add_param('certCaContent', certCaContent)

    def get_certCaId(self):
        return self.get_params().get('certCaId')

    def set_certCaId(self, certCaId):
        self.add_param('certCaId', certCaId)

    def get_certCaName(self):
        return self.get_params().get('certCaName')

    def set_certCaName(self, certCaName):
        self.add_param('certCaName', certCaName)

    def get_certContent(self):
        return self.get_params().get('certContent')

    def set_certContent(self, certContent):
        self.add_param('certContent', certContent)

    def get_certId(self):
        return self.get_params().get('certId')

    def set_certId(self, certId):
        self.add_param('certId', certId)

    def get_certKey(self):
        return self.get_params().get('certKey')

    def set_certKey(self, certKey):
        self.add_param('certKey', certKey)

    def get_certName(self):
        return self.get_params().get('certName')

    def set_certName(self, certName):
        self.add_param('certName', certName)

    def get_forwardProtocol(self):
        return self.get_params().get('forwardProtocol')

    def set_forwardProtocol(self, forwardProtocol):
        self.add_param('forwardProtocol', forwardProtocol)

    def get_listenerId(self):
        return self.get_params().get('listenerId')

    def set_listenerId(self, listenerId):
        self.add_param('listenerId', listenerId)

    def get_listenerName(self):
        return self.get_params().get('listenerName')

    def set_listenerName(self, listenerName):
        self.add_param('listenerName', listenerName)

    def get_loadBalancerId(self):
        return self.get_params().get('loadBalancerId')

    def set_loadBalancerId(self, loadBalancerId):
        self.add_param('loadBalancerId', loadBalancerId)

    def get_sslMode(self):
        return self.get_params().get('sslMode')

    def set_sslMode(self, sslMode):
        self.add_param('sslMode', sslMode)



# -*- coding: utf-8 -*-

from ..qcloudsdkcore.request import Request

class DisassociateInstancesKeyPairsRequest(Request):

    def __init__(self):
        super(DisassociateInstancesKeyPairsRequest, self).__init__(
            'cvm', 'qcloudcliV1', 'DisassociateInstancesKeyPairs', 'cvm.api.qcloud.com')

    def get_ForceStop(self):
        return self.get_params().get('ForceStop')

    def set_ForceStop(self, ForceStop):
        self.add_param('ForceStop', ForceStop)

    def get_InstanceIds(self):
        return self.get_params().get('InstanceIds')

    def set_InstanceIds(self, InstanceIds):
        self.add_param('InstanceIds', InstanceIds)

    def get_KeyIds(self):
        return self.get_params().get('KeyIds')

    def set_KeyIds(self, KeyIds):
        self.add_param('KeyIds', KeyIds)



# -*- coding: utf-8 -*-

from ..qcloudsdkcore.request import Request

class ResetInstancePasswordRequest(Request):

    def __init__(self):
        super(ResetInstancePasswordRequest, self).__init__(
            'cvm', 'qcloudcliV1', 'ResetInstancePassword', 'cvm.api.qcloud.com')

    def get_hardPowerOffFlag(self):
        return self.get_params().get('hardPowerOffFlag')

    def set_hardPowerOffFlag(self, hardPowerOffFlag):
        self.add_param('hardPowerOffFlag', hardPowerOffFlag)

    def get_instanceIds(self):
        return self.get_params().get('instanceIds')

    def set_instanceIds(self, instanceIds):
        self.add_param('instanceIds', instanceIds)

    def get_password(self):
        return self.get_params().get('password')

    def set_password(self, password):
        self.add_param('password', password)



# -*- coding: utf-8 -*-

from ..qcloudsdkcore.request import Request

class DescribeInstanceInternetBandwidthConfigsRequest(Request):

    def __init__(self):
        super(DescribeInstanceInternetBandwidthConfigsRequest, self).__init__(
            'cvm', 'qcloudcliV1', 'DescribeInstanceInternetBandwidthConfigs', 'cvm.api.qcloud.com')

    def get_InstanceId(self):
        return self.get_params().get('InstanceId')

    def set_InstanceId(self, InstanceId):
        self.add_param('InstanceId', InstanceId)



# -*- coding: utf-8 -*-

from ..qcloudsdkcore.request import Request

class ResizeInstanceDisksRequest(Request):

    def __init__(self):
        super(ResizeInstanceDisksRequest, self).__init__(
            'cvm', 'qcloudcliV1', 'ResizeInstanceDisks', 'cvm.api.qcloud.com')

    def get_DataDisks(self):
        return self.get_params().get('DataDisks')

    def set_DataDisks(self, DataDisks):
        self.add_param('DataDisks', DataDisks)

    def get_DryRun(self):
        return self.get_params().get('DryRun')

    def set_DryRun(self, DryRun):
        self.add_param('DryRun', DryRun)

    def get_ForceStop(self):
        return self.get_params().get('ForceStop')

    def set_ForceStop(self, ForceStop):
        self.add_param('ForceStop', ForceStop)

    def get_InstanceId(self):
        return self.get_params().get('InstanceId')

    def set_InstanceId(self, InstanceId):
        self.add_param('InstanceId', InstanceId)

    def get_Uin(self):
        return self.get_params().get('Uin')

    def set_Uin(self, Uin):
        self.add_param('Uin', Uin)



# -*- coding: utf-8 -*-

from ..qcloudsdkcore.request import Request

class DescribeBmLocationBackendsRequest(Request):

    def __init__(self):
        super(DescribeBmLocationBackendsRequest, self).__init__(
            'bmlb', 'qcloudcliV1', 'DescribeBmLocationBackends', 'bmlb.api.qcloud.com')

    def get_domainId(self):
        return self.get_params().get('domainId')

    def set_domainId(self, domainId):
        self.add_param('domainId', domainId)

    def get_limit(self):
        return self.get_params().get('limit')

    def set_limit(self, limit):
        self.add_param('limit', limit)

    def get_listenerId(self):
        return self.get_params().get('listenerId')

    def set_listenerId(self, listenerId):
        self.add_param('listenerId', listenerId)

    def get_loadBalancerId(self):
        return self.get_params().get('loadBalancerId')

    def set_loadBalancerId(self, loadBalancerId):
        self.add_param('loadBalancerId', loadBalancerId)

    def get_locationId(self):
        return self.get_params().get('locationId')

    def set_locationId(self, locationId):
        self.add_param('locationId', locationId)

    def get_offset(self):
        return self.get_params().get('offset')

    def set_offset(self, offset):
        self.add_param('offset', offset)

    def get_queryType(self):
        return self.get_params().get('queryType')

    def set_queryType(self, queryType):
        self.add_param('queryType', queryType)



# -*- coding: utf-8 -*-

from ..qcloudsdkcore.request import Request

class ModifyBmNatGatewayRequest(Request):

    def __init__(self):
        super(ModifyBmNatGatewayRequest, self).__init__(
            'bmvpc', 'qcloudcliV1', 'ModifyBmNatGateway', 'bmvpc.api.qcloud.com')

    def get_natId(self):
        return self.get_params().get('natId')

    def set_natId(self, natId):
        self.add_param('natId', natId)

    def get_natName(self):
        return self.get_params().get('natName')

    def set_natName(self, natName):
        self.add_param('natName', natName)

    def get_unVpcId(self):
        return self.get_params().get('unVpcId')

    def set_unVpcId(self, unVpcId):
        self.add_param('unVpcId', unVpcId)

    def get_vpcId(self):
        return self.get_params().get('vpcId')

    def set_vpcId(self, vpcId):
        self.add_param('vpcId', vpcId)



# coding: utf-8

from io import open
from setuptools import setup, find_packages

version = '0.1.0'


# FIXME: We temporarily comment these direct url dependencies so that the project can be published to pypi
requirements = [
    'setuptools',
    'six',

    # sa-notify
    # 'wechat @ git+https://github.com/tclh123/wechat.git@py3k#egg=wechat-0.4.17',
    'pycrypto',

    # sa-dns
    # 'pydnspod @ git+https://github.com/tclh123/pydnspod.git@py3k#egg=pydnspod-0.0.3',

    # sa-script
    'ansible>=2.8',
    'progressbar2',
    'terminaltables',

    # sa-access
    'elasticsearch>=2.0.0,<3.0.0',

    # sa-disk
    'humanize',

    # sa-icinga
    'sentry-sdk',
    'Mako',
    # 'icinga2-api @ git+https://github.com/tclh123/icinga2-api.git@master#egg=icinga2-api-0.1.0',
    'inflect',
]


entry_points = """
      [console_scripts]
      sa-uptime = sa_tools_core.uptime:main
      sa-dns = sa_tools_core.dns:main
      # sa-node = sa_tools_core.node:main
      # sa-rsync = sa_tools_core.rsync:main
      sa-script = sa_tools_core.script:main
      # sa-lvs = sa_tools_core.lvs:main
      sa-disk = sa_tools_core.disk:main
      sa-notify = sa_tools_core.notify:main
      sa-icinga = sa_tools_core.icinga:main
      sa-access = sa_tools_core.access:main
      sa-bs = sa_tools_core.bs:main
      """

scripts = [
]

setup(name='sa-tools-core',
      version=version,
      description="SA Tools Core",
      long_description=open("README.md", encoding='utf-8').read(),
      long_description_content_type='text/markdown',
      # Get more strings from
      # https://pypi.python.org/pypi?%3Aaction=list_classifiers
      classifiers=[
          'Programming Language :: Python',
          'Programming Language :: Python :: 2.7',
          'Programming Language :: Python :: 3',
          'Programming Language :: Python :: 3.4',
          'Programming Language :: Python :: 3.5',
          'Programming Language :: Python :: 3.6',
          'Programming Language :: Python :: 3.7',
          'Development Status :: 5 - Production/Stable',
          'Environment :: Console',
          'Intended Audience :: Developers',
          'License :: OSI Approved :: BSD License',
          'Operating System :: POSIX :: Linux',
          'Topic :: Software Development :: Libraries :: Python Modules'
      ],
      url='https://github.com/douban/sa-tools-core',
      keywords=['sa-tools', 'sysadmin', 'command line tools'],
      author='sysadmin',
      author_email='sysadmin@douban.com',
      license='BSD License',
      packages=find_packages(exclude=['examples*', 'tests*']),
      include_package_data=True,
      zip_safe=False,
      entry_points=entry_points,
      scripts=scripts,

      install_requires=requirements,
)  # NOQA



# coding: utf-8

from __future__ import absolute_import

import logging

import requests
from requests.packages.urllib3.exceptions import InsecureRequestWarning
from wechat.enterprise import WxApi

from sa_tools_core.utils import get_config


logger = logging.getLogger(__name__)
requests.packages.urllib3.disable_warnings(InsecureRequestWarning)

# CORPID = 'aaaaaaaaaaaaaaaaaa'
# CORPSECRET = "bbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbb"
# APPID = 100002 or 2


def send_message(touser=None, content=None, safe="0", msg_type='text',
                 toparty=None, totag=None, qy=False, **kwargs):
    if qy:
        secret = get_config('wework')
        safe = None
    else:
        secret = get_config('wechat')
    CORPID, CORPSECRET, APPID = secret.split(':')
    agentid = int(APPID)
    api = WxApi(CORPID, CORPSECRET)

    ret, api_error = api.send_message(msg_type=msg_type,
                                      content=content,
                                      agentid=agentid,  # APP ID
                                      safe=safe,
                                      # 成员ID列表（消息接收者，多个接收者用‘|’分隔，最多支持1000个）。特殊情况：指定为@all，则向关注该企业应用的全部成员发送
                                      touser=touser,
                                      # 部门ID列表，多个接收者用‘|’分隔，最多支持100个。当touser为@all时忽略本参数
                                      toparty=toparty,
                                      # 标签ID列表，多个接收者用‘|’分隔。当touser为@all时忽略本参数
                                      totag=totag,
                                      )
    if api_error:
        logger.error((
            'send_message({touser}, {content}) get {err_code}, {err_msg}'
            .format(touser=touser,
                    content=content,
                    err_code=api_error.code,
                    err_msg=api_error.message)))
    return ret, api_error



# -*- coding: utf-8 -*-

from ..qcloudsdkcore.request import Request

class EipBmDeleteRequest(Request):

    def __init__(self):
        super(EipBmDeleteRequest, self).__init__(
            'bmeip', 'qcloudcliV1', 'EipBmDelete', 'bmeip.api.qcloud.com')

    def get_eipIds(self):
        return self.get_params().get('eipIds')

    def set_eipIds(self, eipIds):
        self.add_param('eipIds', eipIds)



# -*- coding: utf-8 -*-

from ..qcloudsdkcore.request import Request

class ShutdownDeviceRequest(Request):

    def __init__(self):
        super(ShutdownDeviceRequest, self).__init__(
            'bm', 'qcloudcliV1', 'ShutdownDevice', 'bm.api.qcloud.com')

    def get_instanceIds(self):
        return self.get_params().get('instanceIds')

    def set_instanceIds(self, instanceIds):
        self.add_param('instanceIds', instanceIds)

    def get_opUin(self):
        return self.get_params().get('opUin')

    def set_opUin(self, opUin):
        self.add_param('opUin', opUin)



# -*- coding: utf-8 -*-

from ..qcloudsdkcore.request import Request

class ResetInstancesPasswordRequest(Request):

    def __init__(self):
        super(ResetInstancesPasswordRequest, self).__init__(
            'cvm', 'qcloudcliV1', 'ResetInstancesPassword', 'cvm.api.qcloud.com')

    def get_ForceStop(self):
        return self.get_params().get('ForceStop')

    def set_ForceStop(self, ForceStop):
        self.add_param('ForceStop', ForceStop)

    def get_InstanceIds(self):
        return self.get_params().get('InstanceIds')

    def set_InstanceIds(self, InstanceIds):
        self.add_param('InstanceIds', InstanceIds)

    def get_Password(self):
        return self.get_params().get('Password')

    def set_Password(self, Password):
        self.add_param('Password', Password)

    def get_UserName(self):
        return self.get_params().get('UserName')

    def set_UserName(self, UserName):
        self.add_param('UserName', UserName)



# -*- coding: utf-8 -*-

from ..qcloudsdkcore.request import Request

class DescribeEipBmQuotaRequest(Request):

    def __init__(self):
        super(DescribeEipBmQuotaRequest, self).__init__(
            'bmeip', 'qcloudcliV1', 'DescribeEipBmQuota', 'bmeip.api.qcloud.com')



# -*- coding: utf-8 -*-

from ..qcloudsdkcore.request import Request

class DeleteBmSubnetRequest(Request):

    def __init__(self):
        super(DeleteBmSubnetRequest, self).__init__(
            'bmvpc', 'qcloudcliV1', 'DeleteBmSubnet', 'bmvpc.api.qcloud.com')

    def get_subnetId(self):
        return self.get_params().get('subnetId')

    def set_subnetId(self, subnetId):
        self.add_param('subnetId', subnetId)

    def get_unSubnetId(self):
        return self.get_params().get('unSubnetId')

    def set_unSubnetId(self, unSubnetId):
        self.add_param('unSubnetId', unSubnetId)

    def get_unVpcId(self):
        return self.get_params().get('unVpcId')

    def set_unVpcId(self, unVpcId):
        self.add_param('unVpcId', unVpcId)

    def get_vpcId(self):
        return self.get_params().get('vpcId')

    def set_vpcId(self, vpcId):
        self.add_param('vpcId', vpcId)



# -*- coding: utf-8 -*-

from ..qcloudsdkcore.request import Request

class UpdateInstanceBandwidthRequest(Request):

    def __init__(self):
        super(UpdateInstanceBandwidthRequest, self).__init__(
            'cvm', 'qcloudcliV1', 'UpdateInstanceBandwidth', 'cvm.api.qcloud.com')

    def get_bandwidth(self):
        return self.get_params().get('bandwidth')

    def set_bandwidth(self, bandwidth):
        self.add_param('bandwidth', bandwidth)

    def get_endTime(self):
        return self.get_params().get('endTime')

    def set_endTime(self, endTime):
        self.add_param('endTime', endTime)

    def get_instanceId(self):
        return self.get_params().get('instanceId')

    def set_instanceId(self, instanceId):
        self.add_param('instanceId', instanceId)

    def get_startTime(self):
        return self.get_params().get('startTime')

    def set_startTime(self, startTime):
        self.add_param('startTime', startTime)



# -*- coding: utf-8 -*-

from ..qcloudsdkcore.request import Request

class EipBmQueryTaskRequest(Request):

    def __init__(self):
        super(EipBmQueryTaskRequest, self).__init__(
            'bmeip', 'qcloudcliV1', 'EipBmQueryTask', 'bmeip.api.qcloud.com')

    def get_requestId(self):
        return self.get_params().get('requestId')

    def set_requestId(self, requestId):
        self.add_param('requestId', requestId)



from . import (BuyDeviceRequest,DescribeBmAgentStatusRequest,DescribeBmInstancesRequest,DescribeDeviceClassPartitionRequest,DescribeDeviceClassRequest,DescribeDeviceHardwareInfoRequest,DescribeDeviceInventoryRequest,DescribeDeviceOperationLogRequest,DescribeDevicePartitionRequest,DescribeDevicePositionRequest,DescribeDevicePriceRequest,DescribeDeviceRequest,DescribeHardwareSpecificationRequest,DescribeOsRequest,DescribeRegionsRequest,DescriptionOperationResultRequest,GetDeviceDeployProcessRequest,ModifyDeviceAliasRequest,ModifyDeviceAutoRenewFlagRequest,ModifyPayModePre2PostRequest,OfflineDeviceRequest,QueryElasticPriceRequest,RebootDeviceRequest,ReloadDeviceOsRequest,ResetDevicePasswdRequest,ShutdownDeviceRequest,StartDeviceRequest,)



# -*- coding: utf-8 -*-

from ..qcloudsdkcore.request import Request

class DescribeOsRequest(Request):

    def __init__(self):
        super(DescribeOsRequest, self).__init__(
            'bm', 'qcloudcliV1', 'DescribeOs', 'bm.api.qcloud.com')

    def get_deviceClassCode(self):
        return self.get_params().get('deviceClassCode')

    def set_deviceClassCode(self, deviceClassCode):
        self.add_param('deviceClassCode', deviceClassCode)




# -*- coding: utf-8 -*-

from ..qcloudsdkcore.request import Request

class InquiryPriceResetInstancesInternetMaxBandwidthRequest(Request):

    def __init__(self):
        super(InquiryPriceResetInstancesInternetMaxBandwidthRequest, self).__init__(
            'cvm', 'qcloudcliV1', 'InquiryPriceResetInstancesInternetMaxBandwidth', 'cvm.api.qcloud.com')

    def get_DryRun(self):
        return self.get_params().get('DryRun')

    def set_DryRun(self, DryRun):
        self.add_param('DryRun', DryRun)

    def get_EndTime(self):
        return self.get_params().get('EndTime')

    def set_EndTime(self, EndTime):
        self.add_param('EndTime', EndTime)

    def get_InstanceIds(self):
        return self.get_params().get('InstanceIds')

    def set_InstanceIds(self, InstanceIds):
        self.add_param('InstanceIds', InstanceIds)

    def get_InternetAccessible(self):
        return self.get_params().get('InternetAccessible')

    def set_InternetAccessible(self, InternetAccessible):
        self.add_param('InternetAccessible', InternetAccessible)

    def get_StartTime(self):
        return self.get_params().get('StartTime')

    def set_StartTime(self, StartTime):
        self.add_param('StartTime', StartTime)



# -*- coding: utf-8 -*-

from ..qcloudsdkcore.request import Request

class ModifyInstanceAttributesRequest(Request):

    def __init__(self):
        super(ModifyInstanceAttributesRequest, self).__init__(
            'cvm', 'qcloudcliV1', 'ModifyInstanceAttributes', 'cvm.api.qcloud.com')

    def get_instanceId(self):
        return self.get_params().get('instanceId')

    def set_instanceId(self, instanceId):
        self.add_param('instanceId', instanceId)

    def get_instanceName(self):
        return self.get_params().get('instanceName')

    def set_instanceName(self, instanceName):
        self.add_param('instanceName', instanceName)



# -*- coding: utf-8 -*-

from ..qcloudsdkcore.request import Request

class ModifyDeviceAutoRenewFlagRequest(Request):

    def __init__(self):
        super(ModifyDeviceAutoRenewFlagRequest, self).__init__(
            'bm', 'qcloudcliV1', 'ModifyDeviceAutoRenewFlag', 'bm.api.qcloud.com')

    def get_autoRenewFlag(self):
        return self.get_params().get('autoRenewFlag')

    def set_autoRenewFlag(self, autoRenewFlag):
        self.add_param('autoRenewFlag', autoRenewFlag)

    def get_instanceIds(self):
        return self.get_params().get('instanceIds')

    def set_instanceIds(self, instanceIds):
        self.add_param('instanceIds', instanceIds)



# -*- coding: utf-8 -*-

from ..qcloudsdkcore.request import Request

class DescribeRegionsRequest(Request):

    def __init__(self):
        super(DescribeRegionsRequest, self).__init__(
            'bm', 'qcloudcliV1', 'DescribeRegions', 'bm.api.qcloud.com')

    def get_regionId(self):
        return self.get_params().get('regionId')

    def set_regionId(self, regionId):
        self.add_param('regionId', regionId)
